function [lambdaX,lambda50,lambdam] = CheckMAF( x,Sa,bSa,T1,scalemethod,sf,MAFDS,betaepist)
% [lambdaX,lambda50,lambdam] = CheckMAF( x,Sa,bSa,T1,scalemethod,sf,MAFDS,betaepist )
% Estimates the Mean Annual Frequency of exceeding the Performance Objective
% -------------------------------------------------------------------------
% INPUT: 
% x           = confidence level
% Sa          = median value of the lognormal fragility curve
% bSa         = variation (i.e. standard deviation of the log data) of the 
%               fragility curve 
% T1          = period of the bldg 
% scalemethod = scalemethod=0: No scaling of the Hazard Curve (Default)
%               scalemethod=1: Scale the MAF of the hazard, not the Sa 
%               values. It makes more sense as it corresponds to making the
%               fault(s) less active by decreasing their seismic rate but 
%               without changing anything else. This is also the kind of
%               scaling DV is implementing in the YFSapp.
% sf          = scale factor of the HC
% MAFDS       = MAF capacity of the PO
% betaepist   = additional uncertainty
% -------------------------------------------------------------------------
% OUTPUT
% lambdaX     = x% confidence estimate of the MAF
% lambda50    = median estimate of the MAF
% lambdam     = mean estimate of the MAF
% -------------------------------------------------------------------------
if nargin<7; betaepist=0.0; end
if nargin<4; scalemethod=0; end
if or(nargin<5,isempty(sf)); sf=1.; end


hz=load('SanJose.hz.mat');
[IM,MAF]=get_SaHazardcurve(hz,T1,0);

% scale the hazard 
switch scalemethod
   case 1
      % fit a spline to the discrete points
      %indy=find(MAF~=0);
      %spline_loghz=spline(log(IM(indy)),log(MAF(indy)));
      %MAFDL_VanNuys=exp(ppual(spline_loghz,log(SaDL)));
      %sf=MAFDL/MAFDL_VanNuys;
      fprintf('MAF-based hazard scale factor %0.4g\n',sf)
      sMAF=MAF*sf;
      sIM=IM;
   case 2
      % fit a spline to the discrete points
      %indy=find(MAF~=0);
      %ispline_loghz=spline(log(MAF(indy)),log(IM(indy)));
      %SaDL_VanNuys=exp(ppual(ispline_loghz,log(MAFDL)));
      %sf=SaDL/SaDL_VanNuys;
      fprintf('Sa-based hazard scale factor %0.4g\n',sf)
      sMAF=MAF;
      sIM=IM*sf;
   otherwise
      sMAF=MAF;
      sIM=IM;
      fprintf('No scaling of hazard performed\n')
end



bTSa=sqrt(betaepist^2+bSa^2);
% Use the scaled MAFs here, to be compatible with code @ DL.
% Median estimate vis-a-vis uncertainty (uncertainty is disregarded)
[lambda50,extrapl]=MAF_of_limitstate_basedon_IM_hazfast([log(Sa);bSa],sIM,sMAF);
%  Mean estimate vis-a-vis uncertainty (uncertainty is included)
[lambdam,extrapl]=MAF_of_limitstate_basedon_IM_hazfast([log(Sa);bTSa],sIM,sMAF);

% x% confidence estimate
nSa=logninv(1-x,log(Sa),betaepist);
[lambdaX,extrapl]=MAF_of_limitstate_basedon_IM_hazfast([log(nSa);bSa],sIM,sMAF);

if extrapl~=0, warning('We have extrapolation issues'); end

% Check
fprintf('Median        MAF @ DL = %8.4f\n',lambda50)
fprintf('Max allowable MAF @ DL = %8.4f\n',MAFDS)
fprintf('with confidence MAF @ DL = %8.4f\n',lambdaX)



end

