function [sa,nMAF]=get_SaHazardcurve(hz,nT,plotflag)
% Will perform interpolation of given hazard curves to provide
% new sa-hazard curves for arbitrary period nT within the available period
% and Sa data 
%_________________________________________________________________________
% INPUT
%  hz  : structure loaded from .hz.mat file. Contains fields T,sa,MAF.
%  nT  : period in seconds
%  plotflag: 1 to plot the hazard curve, 0 not to (default)
% OUTPUT
%  sa  : Sa(nT,5%) values in g's
% nMAF : Mean Annual Frequencies for the corresponding sa-values. 
%__________________________________________________________________________
% Example for LA9 in "Applied Incremental Dynamic Analysis"
% Capacities expressed in IM
% c_Sa=[1.0405 0.5294 0.4078 0.5669 1.1917 1.2254 0.9547 1.1307 0.7179 ...
%       0.6403 0.8218 1.3550 0.1328 0.7194 1.6506 0.3800 1.1896 1.5636 ...
%       0.8458 0.6448];
% hz=load([getdvdir,'/hazard/VanNuys.hz.mat'])
% [sa,nMAF]=get_SaHazardcurve(hz,2.3693);
% % compare results to 
% [sa,nMAF]=VanNuys_Sa_hazard3(2.3693);
% 
% [lambda]=MAF_of_limitstate_basedon_IM(c_Sa,sa,nMAF)
% [lambda,bci,stderr,bci_diff]=MAF_byIM_boot(c_Sa,sa,nMAF)
%
% Created 05/Jun/2016, Copyright (c) D. Vamvatsikos
%         used VanNuys_Sa_hazard3 as a template

if nargin<3, plotflag=0; end
if nargin<2, error('Need at least two arguments'); end
if nT<0 || nT>5, error('Period must be within [0,5sec]'); end

% Definitions of sa, T not needed below as new hz.mat files contain the
% needed data already.
% from Norm's input file in cm/s2, but converted to g's here!
% sa=[0.01,0.02,0.03,0.04,0.05,0.075,0.10,0.15,0.20,0.25,0.30,0.40,0.50,0.60,0.70,...
%     0.80,0.90,1.00,1.10,1.20,1.30,1.40,1.60,1.80,2.00,...
%     2.20,2.40,2.60,2.80,3.00,3.50,4.00,4.50,5,6,7,8];
% the periods from the attenuation law AS_95b_H in seconds
% T=[0.00, 0.03, 0.04, 0.05, 0.06, 0.075, 0.09, 0.10, 0.12,...
%    0.15, 0.17, 0.20, 0.24, 0.30, 0.36,  0.40, 0.46, 0.50,...
%    0.60, 0.75, 0.85, 1.00, 1.50, 2.00,  3.00, 4.00, 5.00];

MAF=hz.MAF;
sa=hz.Sa;
T=hz.T;


if plotflag
   % Do some proper plotting. Remove all Sa values for which the hazard is 0.
   figure(1); clf; clear indx
   for i=1:length(T)
      indx{i}=find(MAF(i,:)~=0);
      semilogy(sa(indx{i}),MAF(i,indx{i}),'bo-');
      hold on
   end
   grid on
   % Show the relation of the MAF with period, for different levels of SA
   figure(2);clf;
   for j=1:length(sa)
      semilogy(T,MAF(:,j),'bo-');
      hold on
   end
end
% So, observing the above figure, the relation between log(MAF) and 
% period looks linear enough for all SA-levels. Linear
% interpolation will be adequate, between existing T-period curves
% to get hazard curve MAFs for other periods not existing in our database.
i1=max(find(T<nT));
% if nT equals one of the periods for which we do have a ready hazard
% curve then the "equal" below will make sure we get the right result.
% Then of course i1 will be empty, so take care of this below.
i2=min(find(T>=nT));
if isempty(i1)
   %indy=find(MAF(i2,:)~=0);
   nMAF=MAF(i2,:);
else
   indy=find(MAF(i2,:)~=0 & MAF(i1,:)~=0);
   % this will produce warnings as we take the log of some zeros. Ignore them!
   warning off
   lMAF=log(MAF);
   warning on

   for j=1:length(indy)
      nMAF(j) = exp(lMAF(i1,j) + (lMAF(i2,j)-lMAF(i1,j))/...
         (T(i2)-T(i1))*(nT-T(i1)));
   end
   % make all those "inf" into zeros
   nMAF(j+1:length(sa))=zeros(1,length(sa)-length(indy));
   % look at them side by side and verify.
   %[MAF(i1,:)',nMAF',MAF(i2,:)']
end



if plotflag
   figure(1); clf; clear indx
   for i=1:length(T)
      indx{i}=find(MAF(i,:)~=0);
      loglog(sa(indx{i}),MAF(i,indx{i}),'bo-');
      hold on
   end
   grid on
   indy=find(nMAF~=0);
   loglog(sa(indy),nMAF(indy),'ro-');
end



