function plotSection(sectionfname,barRplotfactor,fillflag,matcolor,edgecolor)
% plotSection(sectionfname,barRplotfactor,fillflag,matcolor,edgecolor,figureNumber)
%_________________________________________________________________________
% INPUT
% sectionfname : filename containing section info
% barRplotfactor: determines a magnification factor for plotting fibers
%                 produced via the "layer straight" command. Helps visualize 
%                 the rebars (default = 1)
% fillflag      : If 1, then the fibers are plotted as filled
%                 polygons/circles (default = 0)
% matcolor      : a cell array containing color definitions for each
%                 material (default = {'y','b','r','g','m','k'} )
% edgecolor     : same as above for the edges of the fibers. Used only when
%                 fillflag==1 (default = {'k'})
%
%__________________________________________________________________________
% 
% Helps with taking FiberSection output (actually generation commands) 
% and recreating the section fibers for visualization
% 
% Examples
%__________________________________________________________________________
% plotSection('\users\divamva\opensees\divamva\Kavala\hollowRectfiberS1.out')
% plotSection('\users\divamva\opensees\divamva\Kavala\hollowRectfiberS1.out',2,1,{[0.7,0.7,0.7],[0.5,0.5,0.5]})
% plotSection('\users\divamva\opensees\divamva\Kavala\hollowRectfiberS1.out',2,1,{[0.7,0.7,0.7]},{[0.7,0.7,0.7]})
% plotSection('\users\divamva\opensees\divamva\RCsect\BGRCfiberS2.out')
% plotSection('\users\divamva\opensees\divamva\CEE545\CHSsectionfiberS10.out')
% plotSection('\users\divamva\opensees\divamva\RCsect\RCsectionfiberS3.out')
%__________________________________________________________________________
% Created by D.Vamvatsikos
% 
% Upgraded 07/May/05: Now the first line in the file is not discarded
%                     unless it contains an invalid command (e.g. a title)
%                     Also "support" is provided for "fiber", "layer circ" and
%                     "layer patch". A check is made to ensure "patch quad"
%                     coordinates create a valid convex and counterclockwise
%                     quadrangle!
% Upgraded 07/Oct/05: Added two extra checks for colinearity which were
%                     missing (must check all combinations of size 3)
% Updated  04/Nov/07: Added true support for the "fiber" command of Opensees.
%                     Also fixed a small issue in the "layer straight"
%                     command where the first coord read was called "z"
%                     instead of "Y". The figure was fine still, because I
%                     had also inverted the plotting of y,z so it came out
%                     ok in the end.
% Updated 18/Sep/12: Added support for "layer circ" and made "patch circ"
%                    work with or without "startang endang" arguments.
% Updated 24/May/17: Added support for "patch rect" 
%_____________________________________________________________________________
% NOTES: 
% 1. Opensees will run even if a clockwise quad is supplied but then weird
% things will happen! Usually axial compression becomes tension and vice-versa
% 2. use view(90,90) to rotate the axes by 90deg and maintain nice labels!
%    This is useful for plotting, e.g. the G9 or G7 sections.



%cd e:\users\divamva\opensees\divamva\RCsect
if nargin<5, edgecolor={'k'}; end
if nargin<4, matcolor={'y','b','r','g','m','k'}; end
if nargin<3, fillflag=0; end
if nargin<2
  % Do not increase bar radius plotting by default
  barRplotfactor=1;
end
fid=fopen(sectionfname);

if fid>=0
   % Since we do not use this "name" at all, refrain from reading it.
   % If the user has not supplied such a name, then the valuable first
   % line will be discarded!
   %name = fgetl(fid);
   i=0;
   while 1
      tline = fgetl(fid);
      % check for EOF. Fully compatible with Matlab 7!
      if ~ischar(tline), break, else i=i+1; end
      % read fiber generation commands
        [firstcom,count,err,ni]=sscanf(tline,'%s',1);
        % only the "fiber" command has a single word as declaration.
        % the rest use two words (e.g. "patch circ")
        if strcmp(lower(firstcom),'fiber')
           % one word was enough. Then numbers follow
           gencom(i).com=firstcom;
        else
           % two words are needed.
           [gencom(i).com,count,err,ni]=sscanf(tline,'%s',2);
        end
        gencom(i).nums=sscanf(tline(ni:end),'%f');
   end
   fclose(fid);
else
!!   error(sprintf('File "%s" not found\n',sectionfname))
end

hold on

for i=1:length(gencom)
   switch lower(gencom(i).com)
     case {'patchquadr','patchrect'}
        tmpcell=num2cell(gencom(i).nums);
        if lower(gencom(i).com(6:7))=='qu'
           % 'patchquadr'
           [mcolor,nIJ,nJK,Iy,Iz,Jy,Jz,Ky,Kz,Ly,Lz]=deal(tmpcell{:});
        else 
           % 'patchrect'
           [mcolor,nIJ,nJK,Iy,Iz,Ky,Kz]=deal(tmpcell{:});
           % fill in missing "Jy,Jz,Ly,Lz" for compatibility with patchquad
           Jy=Ky;
           Jz=Iz;
           Ly=Iy;
           Lz=Kz;
        end
        % check whether they are (1)counterclockwise and (2)form a convex
        % quadrilateral in the y,z coord system
        %
        %  z          L*     *K
        %  |
        %  |        I*       *J
        %  |
        %  +-----y
        %
        % We will achieve this by calculating the outer products of
        % the vectors IJ x IK and IK x IL. If both are positive, 
        % we have convex and counter-clockwise quadrangle.
        % Since all vectors have no x-component, the outerproduct
        % is always along the x-axis and it is calculated as
        % a x b = | ya  za | * (unit x-vector)
        %         | yb  zb |
        % where ya,za are the vector coords (ya=ya2-ya1, za=za2-za1)
        outIJxIK=(Jy-Iy)*(Kz-Iz) - (Ky-Iy)*(Jz-Iz);
        outIKxIL=(Ky-Iy)*(Lz-Iz) - (Ly-Iy)*(Kz-Iz);
        % checking the above is enough to prove convexity. But the one
        % below is needed to check whether points I,J,L are colinear!
        outIJxIL=(Jy-Iy)*(Lz-Iz) - (Ly-Iy)*(Jz-Iz);
        outJKxJL=(Ky-Jy)*(Lz-Jz) - (Ly-Jy)*(Kz-Jz);
        % if they any product is less than zero, then IJKL have been defined
        % clockwise or this is a non-convex quad. If any of them is zero
        % then three points at least are colinear! 
        if outIJxIK<=0 | outIKxIL<=0 | outIJxIL <=0 
           warning(sprintf('Patch quad is non-convex or counter-clockwise defined or has at least 3 colinear points in line %d',i))
        end   
        IJz=linspace(Iz,Jz,nIJ+1);
        IJy=linspace(Iy,Jy,nIJ+1);
        JKz=linspace(Jz,Kz,nJK+1);
        JKy=linspace(Jy,Ky,nJK+1);
        LKz=linspace(Lz,Kz,nIJ+1);
        LKy=linspace(Ly,Ky,nIJ+1);
        ILz=linspace(Iz,Lz,nJK+1);
        ILy=linspace(Iy,Ly,nJK+1);
        if fillflag
           Z=zeros(nIJ+1,nJK+1);
           Y=Z;
           for j=1:(nIJ+1)
              Z(j,:)=linspace(IJz(j),LKz(j),nJK+1);
              Y(j,:)=linspace(IJy(j),LKy(j),nJK+1);
           end
           for j=1:nIJ
              for k=1:nJK
                 patch([Z(j,k),Z(j,k+1),Z(j+1,k+1),Z(j+1,k)],...
                    [Y(j,k),Y(j,k+1),Y(j+1,k+1),Y(j+1,k)],...
                    matcolor{1+mod(mcolor,length(matcolor))},...
                    'edgecolor',edgecolor{1+mod(mcolor,length(edgecolor))})
              end
           end
        else
           for j=1:(nIJ+1)
              plot([IJz(j) LKz(j)],[IJy(j) LKy(j)],'color',...
                 matcolor{1+mod(mcolor,length(matcolor))})
           end
           for j=1:(nJK+1)
              plot([JKz(j) ILz(j)],[JKy(j) ILy(j)],'color',...
                 matcolor{1+mod(mcolor,length(matcolor))})
           end
        end
            
    case 'layerstraight'
        tmpcell=num2cell(gencom(i).nums);
        [mcolor,nbars,barA,Iy,Iz,Jy,Jz]=deal(tmpcell{:});
        Y=linspace(Iy,Jy,nbars);
        Z=linspace(Iz,Jz,nbars);
        Radius=sqrt(barA/pi);
        %plot(Y,Z,'ko','markerfacecol','k')
        % plot_circle is better because it captures accurately the area of
        % the bar and does not rescale when we zoom in!
        plot_circle(Z,Y,Radius*barRplotfactor,'','k')
      case 'fiber'
         tmpcell=num2cell(gencom(i).nums);
         [Y,Z,barA,mcolor]=deal(tmpcell{:});
         Radius=sqrt(barA/pi);
         %plot(Y,Z,'ko','markerfacecol','k')
         % plot_circle is better because it captures accurately the area of
         % the bar and does not rescale when we zoom in!
         plot_circle(Z,Y,Radius*barRplotfactor,'','k')
      case 'layercirc'
         %warning('"layer circ" is not supported yet!')
         tmpcell=num2cell(gencom(i).nums);
         if length(tmpcell)==6, tmpcell{7}=0;tmpcell{8}=360; end
         [mcolor,nbars,barA,yC,zC,midR,startang,endang]=deal(tmpcell{:});
         theta=linspace(startang,endang,nbars)*pi/180;
         Z=midR*cos(theta);
         Y=midR*sin(theta);
         Radius=sqrt(barA/pi);
         plot_circle(Z,Y,Radius*barRplotfactor,'','k')
      case 'patchcirc'   
         tmpcell=num2cell(gencom(i).nums);
         if length(tmpcell)==7, tmpcell{8}=0;tmpcell{9}=360; end
         [mcolor,nfp,nft,yC,zC,intR,extR,startang,endang]=deal(tmpcell{:});
         % NOTE: I have not implemented fillflag yet!!!
         Rvals=linspace(intR,extR,nft+1);
         % this plots all the "along thickness" arcs. Use nfp+1 or 2*nfp+1
         % segments to approximate the circle. In this way, the radial lines 
         % plotted below will line up exactly with the approximate circle (polygon actually)
         % Careful, using nfp+1 will make it look too much like a polygon. 
         plot_circular_arc(repmat(zC,1,nft+1),repmat(yC,1,nft+1),Rvals,startang,endang,...
                '-',matcolor{1+mod(mcolor,length(matcolor))},2*nfp+1)
         % this plots all radial lines
         thetavals=linspace(startang*pi/180,endang*pi/180,nfp+1);
         zInt=zC+cos(thetavals)*intR;
         zExt=zC+cos(thetavals)*extR;
         yInt=yC+sin(thetavals)*intR;
         yExt=yC+sin(thetavals)*extR;
         plot([zInt;zExt],[yInt;yExt],'color',matcolor{1+mod(mcolor,length(matcolor))})
    end
    %pause
end

% for box-girder of G9 bridge!
%axis([-15,1,-4,1])
%set(gca,'xdir','reverse')
axis equal
axis tight
set(gca,'box','on')
xlabel('z-coord (m)')
ylabel('y-coord (m)')