# Generates the shear wall model.

# Created 08/07/2019 by A. Chatzidaki & D. Vamvatsikos

wipe
source ../util/GetPeriodSetDamping.tcl
source ../util/units_constants_metric.tcl
source ../util/RunGravityAnalysis.tcl


# Generate a 2D model with 3 dofs per node
model basic -ndm 2 -ndf 3

#-----------------------------------------------------------------------
# Load pattern style to be applied. Default is first mode
# 1 == 1st mode
# 2 == uniform
# 3 == a*x^k
#      x is the height from ground and a is a normalizing constant
#      if k=0 you get a uniform. If k=1 you get a linear pattern etc.
#      In here, use pushoverLPk to define k.
set LPstyle 1
set pushoverLPk 2.0

#------------------------------------------------------------------------
# Offset for node and column numbers for every successive story. This allows for almost 100 bays
set cnod 100


#------------------------------------------------------------------------
# Source the model parameters either from a standard or a new file
if {[file exists "MatIDA_paramfile.tcl"]} {
   puts "Using EXTERNAL parameter file for optimization (Matlab-generated)"
   source MatIDA_paramfile.tcl
} else {
   puts "Using STANDARD parameter file"
   source standard_paramfile_Shear_Wall.tcl
}

#---------------------------------------------------------------------------
# Source the analysistype information. Better to have it in a distinct file.
if {[file exists "Mat_anlsfile.tcl"]} {
   puts "Reading EXTERNAL Analysistype file (Matlab-generated)"
   source Mat_anlsfile.tcl
} else {
	puts "Using STANDARD analysistype file"
	source standard_anlsfile.tcl
}

#---------------------------------------------------------------------------
# Source the timehistory information to apply. The default
# file applies "HDA255.AT2.txt", unscaled
# if another exists, then it is treated as if generated to run an IDA.
if {$analysistype=="dynamic" & [file exists "MatIDA_thfile.tcl"]} {
	set runIDAflag 1
	puts "Using EXTERNAL Timehistory data-file (Matlab-generated)"
	source MatIDA_thfile.tcl
} else {
	set runIDAflag 0
	puts "Using STANDARD Timehistory data-file"
	source standard_thfile.tcl
}
#---------------------------------------------------------------------------
# max allowed subdivisions of dt (for variable transient) or standard subdivisions (for Transient)
set dtsub 1.0

#---------------------------------------------------------------------------
# Eigenvalues estimation
# Estimate up to 3 eigenvalues. 
if {$Nstory<3} {
	set Neigen $Nstory  
} else {
	set Neigen 3 	
}

#________________________________
# Define wall section materials
#________________________________

#~~~~~~ CONCRETE ~~~~~~
# WALL SECTIONS --> 3 different constitutive "laws" for fibers (unconfined concrete 
# for cover, confined concrete for the confined core of the hidden columns, semi-confined 
# concrete for the middle segment)

# ----------------  Confined Core for Hidden Columns --------------------
set fco [expr -38.*$MPa] ; # expected unconfined strength (38 MPa for C30)
set fcc [expr -57.0*$MPa]   ;# confined stress of concrete: estimated at 1.5*fce according to Mander's model
set ecc -0.0070; # confined concrete strain for max compressive strength
set ecu -0.036; # ultimate concrete strain
set eco -0.0020; # unconfined concrete strain for max strength of fc
set esp -0.0076;  # unconfined concrete spalling strain 
set Ec [expr 29350.*$MPa]
set ft [expr -0.1*$fcc]; # concrete tensile strength, about 10% of compressive
#set et [expr $ft/$Ec];   # ORIGINAL concrete tensile strain at max strength
set et [expr 5.*$ft/$Ec];   # 21/11/2017 change to improve stability

set xp 2. ; # Non-dimensional term that defines the strain at which the straight line descent begins in tension, proposed value 2.
set xn 30. ; # Non-dimensional term that defines the strain at which the straight line descent begins in compression, proposed value 30.
set r 1.38  ; # Parameter that controls the nonlinear descending branch estimated by (Ec*ecc/fcc)/((Ec*ecc/fcc)-1.)

#uniaxialMaterial Concrete07 $matTag $fc $ec $Ec $ft $et $xp $xn $r
uniaxialMaterial Concrete07 1 $fcc $ecc $Ec $ft $et $xp $xn $r

#  ----------------  Semi-confined middle segment -------------------- 
set fcc [expr -43.0*$MPa]   ; # estimated at 1.13*fce = 1.13*38
set ecc -0.003; # confined concrete strain for max compressive strength
set ecu -0.02; # concrete strain at crushing strength
set Ec [expr 29350.*$MPa]; # Young's modulus of concrete
set ft [expr -0.1*$fcc]; # concrete tensile strength, about 10% of compressive
#set et [expr $ft/$Ec];   # ORIGINAL concrete tensile strain at max strength
set et [expr 5.*$ft/$Ec];   # 21/11/2017 change to improve stability

#uniaxialMaterial Concrete04 $matTag $fc $ec $ecu $Ec <$fct $et> <$beta>
uniaxialMaterial Concrete04 5 $fcc $ecc $ecu $Ec $ft $et 0.1

#  ----------------  Unconfined concrete for cover ---------------------------
# uniaxialMaterial Concrete01 $matTag $fpc $epsc0 $fpcu $epsU
uniaxialMaterial Concrete01  2  $fco [expr -2.*$fco/$Ec]  0.  [expr -4.*$fco/$Ec];



#~~~~~~ REINFORCING STEEL ~~~~~~
set fys [expr 575.*$MPa]; # steel yield stress
set fus [expr 680.*$MPa]; # ultimate steel stress

set eys [expr $fys/(200.0*$GPa)]; # steel yield strain 			

uniaxialMaterial Hysteretic  40 [expr   $fys]  $eys [expr  $fus] 0.15 [expr  $fus*1.01]  0.16 \
								[expr  -$fys] -$eys [expr (-$fys-0.005/$eys*0.01*$fys)] -0.05 [expr  -100.0*$MPa]  -0.1 \
								0.5 0.5 0 0 0		

uniaxialMaterial MinMax 4 40 -min -0.20 -max 0.20



#________________________________
# Genetate wall section 
#________________________________

# Basic Assumptions: 
# While in most cases, each of the wall section's cross-section doesn't change along the height,
# different id's are generated for each floor for the sake of clarity and customizability 

# wall section id's at each level 
set Csect " "
for {set i 1} {$i<=$Nstory} {incr i} {
	# generating id array for wall sections of the form [11 21 31 41...nth_floor*10+1]
	lappend Csect [expr 10*$i+1]
}


# ~~~~~~~ WALL SECTION INPUT ~~~~~~~~~~~

# Define number of fibers along y (horizontal) and z (vertical) axes for cover patches, confined patches
# and unconfined (mid) patch
set nfcovY 5 ; #number of fibers along y axis for cover patches
set nfcovZ 1;   #number of fibers along z axis for cover patches (1 since this affects mostly the edge vertical patches)
set nfconY 15 ; #number of fibers along y axis for confined patches
set nfconZ 1 ; #number of fibers along z axis for confined patches
set nfuncY [expr int(10*$cwlu)] ; #number of fibers along y axis for the unconfined segment between the hidden columns : 10 times the length of the middle segment.
   
# Define rebar patterns for unconfined segment and hidden columns
   
# Unconfined segment longitudinal rebar pattern
set nlayers 13 ; #number of vertical layers (pair of rebars per layer usually) along the segment length
set lspacing [expr 0.3*$m] ; #spacing between the layers
set unbarA [expr 0.000314*$m2]  ; #assuming each rebar is 20 in the segment
   
# Hidden columns
set hrlay 5 ; #how many rebar (vertical) layers exist within one hidden column
set barArea [expr 0.000804*$m2] ; # 32 for each rebar in the hidden columns

# feed total length, width and length respectively of the unconfined segment 
# for the wall section from the standard paramfile
set lt $cwlen ; # total length of the wall section
set bt $cwb  ; # width of the wall section
set lu $cwlu ; # length of the middle segment

#source ../RCSect/RCsectionwallShearWall.tcl; # the steel rebars are placed at their location
source ../RCSect/RCsectionwallShearWall2.tcl; # the rebars are concentrated in the center of the wall


for {set i 1} {$i<=$Nstory} {incr i} {
	set i_1 [expr $i-1]
	# GENERATING WALL SECTION
	set wallid [lindex $Csect $i_1]
	#RCsectionwall {id   l  b   lu cover coreID coverID steelID midcID hrlay barArea $nfconY $nfconZ $nfcovY $nfcovZ $nfuncY $nlayers $lspacing $unbarA {MatlabOut 0}}
	RCsectionwallShearWall $wallid $lt $bt $lu $cwc 1 2 4 5 $hrlay $barArea $nfconY $nfconZ $nfcovY $nfcovZ $nfuncY $nlayers $lspacing $unbarA 1   
}

#---------------------------
# define transformation: linear for the pd column and pdelta for the wall
# for pd column
geomTransf Linear 1
# for columns
geomTransf PDelta 2
 
# number of integration points
set nP 5

#________________________________
# Node generation
#________________________________

# Generate ground (support) nodes and fix them 
set npat 2  ; 

for {set i 1} {$i<=$npat-1} {incr i} {
	node $i 0.0 0.0
	if {$fixedbase==1} {
		fix $i 1 1 1
	} else {
		fix $i 1 1 0
	}
}

# If PDcolumn flag ==1 generate PDcolumn ground node and pin it
if {$PDcolumn==1} {
	node $npat [expr 3.] 0.0; # distance between PDcolumn node and the rightmost node of the structure is 3m
	fix $npat 1 1 0 ; # pin pd column
}


# FLOOR NODES (PD COLUMN INCLUDED)
set tHeight 0.0

for {set i 1} {$i<=$Nstory} {incr i} {
	set i_1 [expr $i-1]
	set tHeight [expr $tHeight +[lindex $storyHeight $i_1]]
	# generate mass array for each floor node  - 2 nodes per floor: one of the wall and one of the leaning column
	set mnod [expr [lindex $floormass $i_1]*$pTrib] ; # mass that goes to the nodes 
	set mpd [expr [lindex $floormass $i_1]*(1.-$pTrib)] ; # mass of the leaning column 
	set tMass "$mnod $mpd"
	for {set j 1} {$j<=$npat} {incr j} {
		set j_1 [expr $j-1]
		set node_y $tHeight
		set node_m [lindex $tMass $j_1]
		node [expr $i*$cnod+$j] [expr ($j-1)*3.] $node_y -mass $node_m 0. 0.
		
		if {$j>1} {
			equalDOF [expr 1+$i*$cnod] [expr $j+$i*$cnod] 1
		}
	}
}

# add extra nodes because the wall of the 1st story will be discretized into 5 elements
# Not included in the loop above to be crystal clear
node [expr 50001] 0.0 [expr $fsheight/20.]
node [expr 50002] 0.0 [expr 2.*$fsheight/20.]
node [expr 50003] 0.0 [expr 1.*$fsheight/5.]
node [expr 50004] 0.0 [expr 3.*$fsheight/5.]


#________________________________
# Generate wall section elements
#________________________________

for {set i 0} {$i<=$Nstory-1} {incr i} {
	set wallid [lindex $Csect $i]
    # puts "$CsecID"
	
	# for force-based elements, the command below should be used
	# element nonlinearBeamColumn $eleTag $iNode $jNode $numIntgrPts $secTag $transfTag <-mass $massDens> <-iter $maxIters $tol>
	# element  nonlinearBeamColumn [expr $i*$cnod+1]  [expr $i*$cnod+1] [expr ($i+1)*$cnod+1]\
    #       $nP $wallid  2 -iter 20 1e-12
 
	# for displacement-based elements
	if {$i==0} {
		# 5 displacement-based elements in the 1st story because the plasticity is expected to 
		# concentrate at the base of the wall
		# This can be changed by a loop but I prefered to leave it this way to be clear
		element dispBeamColumn [expr 5000+1] 1 50001 $nP [lindex $Csect $i] 2
		element dispBeamColumn [expr 5000+2] 50001 50002 $nP [lindex $Csect $i] 2
		element dispBeamColumn [expr 5000+3] 50002 50003 $nP [lindex $Csect $i] 2
		element dispBeamColumn [expr 5000+4] 50003 50004 $nP [lindex $Csect $i] 2
		element dispBeamColumn [expr 5000+5] 50004 101 $nP [lindex $Csect $i] 2
	} else {
		# one displacement-based element for the rest of the stories
		# element dispBeamColumn $eleTag $iNode $jNode $numIntgrPts $secTag $transfTag
		element dispBeamColumn [expr $i*$cnod+1] [expr $i*$cnod+1] [expr ($i+1)*$cnod+1] $nP $wallid 2 
	}
}



#_______________________
# Generate P-D column
#_______________________
if {$PDcolumn==1} {
	for {set i 0} {$i<=$Nstory-1} {incr i} {

		# considering the inertia properties of 1/4 of the building thus including 2 gravity columns and one shear wall
		# acting in the other direction
		element elasticBeamColumn [expr $i*$cnod+2]  [expr $i*$cnod+2] [expr ($i+1)*$cnod+2]\
                              [expr $NcolPD*[lindex $AcolPD $i]+$cwlen*$cwb]  $Ec [expr $NcolPD*[lindex $IcolPD $i]+$cwlen*$cwb**3./12.]   1
		
	}
}


#_______________________
# Define masses and loads 
#_______________________


#
pattern Plain 1 "Linear" {
	for {set i 1} {$i<=$Nstory} {incr i} {
		set i_1 [expr $i-1]
		if {$PDcolumn==1} {
			set nload [expr -[lindex $floormass $i_1]*$pTrib*$g]
			set nloadPD [expr -[lindex $floormass $i_1]*(1.-$pTrib)*$g]
			set j $npat
			load   [expr $j+$i*$cnod]     0.0  $nloadPD  0.0
		} else {
			set nload [expr -[lindex $floormass $i_1]*$g]	
		}
		for {set j 1} {$j<=$npat-1} {incr j} {
			load   [expr $j+$i*$cnod]     0.0  $nload  0.0
		}
	}
}


#_______________________
# ANALYSIS
#_______________________

constraints Transformation
RunGravityAnalysis 10


# Compute $Neigen modes and apply 1% damping at 1st and 2nd modes 
# NOTE: For fiber models do not apply a lot of damping. Initial cracking is already included. 
# So go up to 1% at most
GetPeriodSetDamping $Neigen 0.01 1 2 "ShearWallPeriods.out"


for {set i 1} {$i<=$Nstory} {incr i} {
	# recorder Drift $fileName <-time> $node1 $node2 $dof $perpDirn

	if {$i<10} {
		# use $i\x.out because $ix.out will search for variable $ix not $i.
		recorder Drift -file idr4f_0$i\x.out -time -iNode [expr ($i-1)*$cnod+0+1] -jNode [expr $i*$cnod+0+1] -dof 1 -perpDirn 2
	} else {
		recorder Drift -file idr4f_$i\x.out -time -iNode [expr ($i-1)*$cnod+0+1] -jNode  [expr $i*$cnod+0+1] -dof 1 -perpDirn 2
	}
}


# roof drift. The dof is along x (1) and the height along y (2)
recorder Drift -file roofdrx.out -time -iNode 1 -jNode [expr $Nstory*$cnod+0+1] -dof 1 -perpDirn 2


# Now allow option to perform either SPO or single dynamic analysis
if {$analysistype=="pushover"} {
	
	################# Display the model #################
	# The default option is to display the model during the pushover analysis. If you want to disable
	# this selection comment the 5 lines below
	recorder display "Model" 10 10 600 600 -wipe
	prp 0 0 50
	vup 0 1 0
	vpn 0 0 1
	display 1 2 10
	
	
	source ../util/RunPushover2Converge.tcl
  
	if {$LPstyle==1} {
		# Create a Plain load pattern with a Linear TimeSeries: Note, the loads below do sum to 1.00!
		set cumsum 0.0
		set totalH 0.0
		for {set i 1} {$i<=$Nstory} {incr i} {
			set i_1 [expr $i-1]
			# use eigenvector 1 in DOF 1 to define the pushover load pattern. Use this loop
			# to find un-normalized forces and then normalize to 1.0 base shear.
			set nodeEigenvector [expr abs([nodeEigenvector [expr $i*$cnod+0+1] 1 1])];
			set pushForce($i) [expr ($nodeEigenvector*[lindex $floormass $i_1])]
			set cumsum [expr $cumsum+$pushForce($i)]
			set totalH [expr $totalH + [lindex $storyHeight [expr $i-1]]]
		}
		puts "Pushover Load Pattern------ 1st mode"
		pattern Plain 2 Linear {
			for {set i 1} {$i<=$Nstory} {incr i} {
				load [expr $i*$cnod+0+1] [expr $pushForce($i)/$cumsum] 0.0 0.0
				puts "[expr $i*$cnod+0+1]: [expr $pushForce($i)/$cumsum]"
			}
		}
	} else {
		set allsum 0.0
		set totalH 0.0
		for {set i 1} {$i<=$Nstory} {incr i} {
			set allsum [expr $allsum + pow($i,$pushoverLPk)]
			set totalH [expr $totalH + [lindex $storyHeight [expr $i-1]]]
		}
		puts "Pushover Load Pattern------ a*x^$pushoverLPk"
		pattern Plain 2 Linear {
			for {set i 1} {$i<=$Nstory} {incr i} {
				load [expr $i*$cnod+0+1] [expr pow($i,$pushoverLPk)/$allsum] 0. 0.
				puts "[expr $i*$cnod+0+1]: [expr pow($i,$pushoverLPk)/$allsum]"
			}
		}	
	}



	#integrator DisplacementControl $nodeTag $dofTag $dU1 <$Jd $minDu $maxDu>
	# 1st step is 0.1% roof drift
	set IDctrlNode [expr $Nstory*$cnod+0+1]
	set DriftStep 0.1e-4
	set DispStep [expr $totalH*$DriftStep]
	set Nsteps 8000
	set DmaxPush [expr $Nsteps*$DispStep]
	puts "Pushing to [expr $Nsteps*$DriftStep*100]% roof drift in $Nsteps steps"
	integrator DisplacementControl $IDctrlNode 1 $DispStep
	#test EnergyIncr 1.0e-5 10 0
	test NormDispIncr 1.0e-8 20 0
  
	#puts " test NormUnbalance 1.0e-6 10"
	#test NormUnbalance 1.0e-6 10
	analysis Static
	RunPushover2Converge  $IDctrlNode $DmaxPush $Nsteps
	loadConst -time 0.0
} elseif {$analysistype=="dynamic"} {
	source ../util/ReadSMDFile3.tcl
	source ../util/RunTransient2Converge2.tcl
	source ../util/DefineXYZ_UniformExcitation.tcl

	# you are getting back "nptsx" and "dtx"
	DefineXYZ_UniformExcitation $dtsub $g $GMdir $GMfileX $GMfileY $GMfileZ $GMfactorX $GMfactorY $GMfactorZ nptsx dtx

	integrator Newmark 0.5 0.25
	test NormDispIncr 1.0e-7 10 0 ; 
	analysis Transient
	puts "analysis Transient"
	RunTransient2Converge [expr $nptsx*$dtsub] [expr $dtx/$dtsub]
}



###28/Juli/2016 I added this command in order to plot the model whichever analysis is run

file delete ShearWall.out 
print ShearWall.out
