# Define a procedure which generates a rectangular reinforced concrete section
# 
# 
# BASED on RCsection.tcl, contained in openSEES website.
# 
#
#
#                   b                                                        VERTICAL REBAR LAYER ENUMERATION
#            COVER PATCHES 1,2,3,4,5,6,7                                            IN HIDDEN COLUMNS
#                                                     z                            
#                            offset                   |                             1    2   3   4
#                            /---/                    |                             |    |   |   |
#               PATCH 3                               |                                  PATCH 6
#         +--!-----------------!----------------------------------------------------!------------------!--+    ~~~~~~~~~~~~~~  Z4     AXONS Zi 
#       P |  !-----------------!                                                    !------------------!  |    ~~~~~~~~~~~~~~  Z3   of reference
#       A |  !                 !                                                    !                  !  |   
#       T |  !                 !                      PATCH 1                       !                  !  |
#       C |  ! o   o  o  o   o !   o        o        o        o        o         o  ! o    o  o  o   o !  | PATCH 7
#       H |  !                 !                                                    !                  !  |-------------->   y   
#         |  !                 !                                                    !                  !  |
#       4 |  !                 !                                                    !                  !  |
#         |  !-----------------!                                                    !------------------!  |    ~~~~~~~~~~~~~~  Z2
#         +--!-----------------!----------------------------------------------------!------------------!--+    ~~~~~~~~~~~~~~  Z1
#         :  :   PATCH 2       :   |        |        |        |        |         |  :      PATCH 5     :  :
#         :  :                 :   |        |        |        |        |         |  :                  :  :
#         :  :                 :   1        2        3        4        5         6  :                  :  :
#         :  :                 :                                                    :                  :  :
#         Y3 Y2                Y1          VERTICAL REBAR LAYER ENUMERATION         Y4               Y5 Y6
#        AXONS Yi of reference                  IN THE MIDDLE SEGMENT
#
# Formal arguments
#    id - tag for the section that is generated by this procedure
#    l - length of the wall section
#    b - overall width of the wall section
#    lu - overall length of the semi-confined wall segment between the two hidden columns
#    cover - thickness of the cover patches
#    coreID - material tag for the core patch
#    coverID - material tag for the cover patches
#    midcID - material tag for the semi-confined wall patches of the middle segment
#    steelID - material tag for the reinforcing steel
#    hrlay - number of (vertical) layers within each one of the hidden columns               
#    barArea - cross-sectional area of each rebar in hidden columns (assuming all rebars have the same x-section) 
#    nfCoreY - number of fibers in the core patch in the y direction (hidden column patch)
#    nfCoreZ - number of fibers in the core patch in the z direction (hidden column patch)
#    nfCoverY - number of fibers in the cover patches with long sides in the y direction
#    nfCoverZ - number of fibers in the cover patches with long sides in the z direction
#    nfmCoverY - number of fibers in the unconfined segment in the y direction
#    relayers - number of vertical rebar layers within then unconfined segment
#    space - spacing between the rebar layers within the unconfined segment
#    unbarA - cross-sectional area of each rebar in the unconfined segment (assuming all rebars have the same x-section) 
#    MatlabOut - if non-zero then the routine sends output to an external file that can be
#                read by matlab to plot the section shape and elements (optional, default=0).
#
# Notes
#    The thickness of cover concrete is constant on all sides of the core.
#    The number of bars is NOT the same on any given side of the section. Only on parallel sides
#    The reinforcing bars are not all the same size. They may differ in y vs z. Note that 
#       the corner position is always taken by bars along z-direction. The bars along y-direction
#       are placed always between the two corners along a length of [h - cover - 2 bar spacings].
#    The number of fibers in the short direction of the cover patches is set to 1.
#
# Created 17/4/2017 by Charis Lyritsakis
# Updated           by Akrivi Chatzidaki: the reinforcement is concentrated in the middle of the section
#                                         since we run a 2D analysis						
# 
proc RCsectionwallShearWall {id l b lu cover coreID coverID steelID midcID hrlay barArea nfCoreY nfCoreZ nfCoverY nfCoverZ nfmCoverY relayers space unbarA {MatlabOut 0}} {


	# hidden column length
	set lh [expr $l/2-$lu/2-$cover]
   
	# axons of reference with respect to the local coordinate system (center of gravity of the wall section)
	# assuming a rectangular wall section (not T or I shaped)
	set Y1 [expr -$lu/2]
	set Y2 [expr -$l/2+$cover]
	set Y3 [expr -$l/2]
	set Y4 [expr -$Y1]
	set Y5 [expr -$Y2]
	set Y6 [expr -$Y3]
   
	set Z1 [expr -$b/2]
	set Z2 [expr -$b/2+$cover]
	set Z3 [expr -$Z2]
	set Z4 [expr -$Z1]

	set Z22 0.; # to place the bars in the center of the wall
	set Z32 0.0; # to place the bars in the center of the wall

	# Define the fiber section
	section fiberSec $id {

		# Define the two confined patches (hidden columns)
		patch rect $coreID $nfCoreY $nfCoreZ $Y2 $Z2 $Y1 $Z3 ; #left confined patch
		patch rect $coreID $nfCoreY $nfCoreZ $Y4 $Z2 $Y5 $Z3 ; #right confined patch
			
		# Define the seven (7) cover patches
		patch rect $midcID $nfmCoverY 1 $Y1 $Z1 $Y4 $Z4 ; #patch 1 (semi-confined segment between hidden columns)
		patch rect $coverID $nfCoverY $nfCoverZ $Y2 $Z1 $Y1 $Z2 ; #patch 2 (horizontal cover,nfCoverZ should be 1)
		patch rect $coverID $nfCoverY $nfCoverZ $Y2 $Z3 $Y1 $Z4 ; #patch 3 (horizontal cover,nfCoverZ should be 1)
		patch rect $coverID 1 $nfCoverZ $Y3 $Z1 $Y2 $Z4 ; #patch 4 (vertical cover,nfCoverZ & nfCoverZ should be 1)
		patch rect $coverID $nfCoverY $nfCoverZ $Y4 $Z1 $Y5 $Z2 ; #patch 5 (horizontal cover,nfCoverZ should be 1)
		patch rect $coverID $nfCoverY $nfCoverZ $Y4 $Z3 $Y5 $Z4 ; #patch 6 (horizontal cover,nfCoverZ should be 1)
		patch rect $coverID 1 $nfCoverZ $Y5 $Z1 $Y6 $Z4 ; #patch 4 (vertical cover,nfCoverZ & nfCoverZ should be 1)

		# Place the rebars in the semi-confined segment
	
		set offset [expr $space/2] ; #distance between the edge of the hidden column and the first layer (assumption is spacing/2)
		set y_0 [expr $Y1+$offset] ; # y0 for the first layer in the unconfined segment (starting from the left side)
	
		for {set i 1} {$i<=$relayers} {incr i} {
			set cy [expr $y_0+($i-1)*$space]
			layer straight $steelID 1 [expr 2.*$unbarA] $cy $Z32 $cy $Z22
		}  
    
		# Place rebars in the hidden columns
		set hrsp [expr $lh/($hrlay-1)] ; #spacing between (vertical) layers in hidden columns
		
		
		# Left hidden column
		layer straight $steelID 1 [expr 3.*$barArea] $Y2 $Z32 $Y2 $Z22
		layer straight $steelID 1 [expr 3.*$barArea] [expr $Y2+$hrsp] $Z32 [expr $Y2+$hrsp] $Z22
		layer straight $steelID 1 [expr 3.*$barArea] [expr $Y2+2*$hrsp] $Z32 [expr $Y2+2*$hrsp] $Z22
		layer straight $steelID 1 [expr 3.*$barArea] [expr $Y2+3*$hrsp] $Z32 [expr $Y2+3*$hrsp] $Z22
		layer straight $steelID 1 [expr 3.*$barArea] [expr $Y2+4*$hrsp] $Z32 [expr $Y2+4*$hrsp] $Z22
	
		# Right hidden column
		layer straight $steelID 1 [expr 3.*$barArea] $Y4 $Z32 $Y4 $Z22
		layer straight $steelID 1 [expr 3.*$barArea] [expr $Y4+$hrsp] $Z32 [expr $Y4+$hrsp] $Z22
		layer straight $steelID 1 [expr 3.*$barArea] [expr $Y4+2*$hrsp] $Z32 [expr $Y4+2*$hrsp] $Z22
		layer straight $steelID 1 [expr 3.*$barArea] [expr $Y4+3*$hrsp] $Z32 [expr $Y4+3*$hrsp] $Z22
		layer straight $steelID 1 [expr 3.*$barArea] [expr $Y4+4*$hrsp] $Z32 [expr $Y4+4*$hrsp] $Z22

		
	}
	if { $MatlabOut == 1 } {
        set matfile [open "RCsectionRectfiberS$id.out" "w"]
		puts $matfile "FiberSection $id"
        puts $matfile "patch rect $coreID $nfCoreY $nfCoreZ $Y2 $Z2 $Y1 $Z3"
		puts $matfile "patch rect $coreID $nfCoreY $nfCoreZ $Y4 $Z2 $Y5 $Z3"
 
		puts $matfile "patch rect $midcID $nfmCoverY 1 $Y1 $Z1 $Y4 $Z4"
		puts $matfile "patch rect $coverID $nfCoverY $nfCoverZ $Y2 $Z1 $Y1 $Z2"
		puts $matfile "patch rect $coverID $nfCoverY $nfCoverZ $Y2 $Z3 $Y1 $Z4"
		puts $matfile "patch rect $coverID 1 $nfCoverZ $Y3 $Z1 $Y2 $Z4"
		puts $matfile "patch rect $coverID $nfCoverY $nfCoverZ $Y4 $Z1 $Y5 $Z2"
		puts $matfile "patch rect $coverID $nfCoverY $nfCoverZ $Y4 $Z3 $Y5 $Z4"
		puts $matfile "patch rect $coverID 1 $nfCoverZ $Y5 $Z1 $Y6 $Z4"
	
		for {set i 1} {$i<=$relayers} {incr i} {
			set cy [expr $y_0+($i-1)*$space]
			puts $matfile "layer straight $steelID 1 [expr 2.*$unbarA] $cy $Z32 $cy $Z22"
		}	  
		# Left hidden column
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] $Y2 $Z32 $Y2 $Z22"
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] [expr $Y2+$hrsp] $Z32 [expr $Y2+$hrsp] $Z22"
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] [expr $Y2+2*$hrsp] $Z32 [expr $Y2+2*$hrsp] $Z22"
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] [expr $Y2+3*$hrsp] $Z32 [expr $Y2+3*$hrsp] $Z22"
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] [expr $Y2+4*$hrsp] $Z32 [expr $Y2+4*$hrsp] $Z22"
	
		# Right hidden column
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] $Y4 $Z32 $Y4 $Z22"
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] [expr $Y4+$hrsp] $Z32 [expr $Y4+$hrsp] $Z22"
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] [expr $Y4+2*$hrsp] $Z32 [expr $Y4+2*$hrsp] $Z22"
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] [expr $Y4+3*$hrsp] $Z32 [expr $Y4+3*$hrsp] $Z22"
		puts $matfile "layer straight $steelID 1 [expr 3.*$barArea] [expr $Y4+4*$hrsp] $Z32 [expr $Y4+4*$hrsp] $Z22"
		
		close $matfile
	}

}
