function PerformanceAssessment_ShearWall(basicdir,AnalysisType,...
                   LabelSize,FigureFontSize,MarkerSize)
% PerformanceAssessment_ShearWall(basicdir,AnalysisType,...
%                    LabelSize,FigureFontSize,MarkerSize)
% -------------------------------------------------------------------------
% Loads the IDA results and performs the checks of Chapter 19 (single
% stripe analysis FDrpo < FC, double stripe analysis FDrpo < FC, and 
% MAFdemand < MAFcapacity. The results and the figures are stored in the
% ResultsPath folder (works for book's example 5). Note that due to the
% cracking of the concrete, the Sa(Teff) should be adopted as the IM
% -------------------------------------------------------------------------
% INPUT
% basicdir:       directory where PerformanceAssessment_ShearWall.m file
%                 is located    
% AnalysisType:   Analysis type:  'single' for single stripe analysis
%                                 'double' for double stripe analysis
%                                 'MAFestimation' to estimate the mean 
%                                 annual frequency (MAF) of exceeding the
%                                 performance objective (PO)
% LabelSize:      size of x and y axes labels of figures
% FigureFontSize: size of fonts in figures
% MarkerSize:     size of the markers plot on IDA curves (default 10)
% -------------------------------------------------------------------------
% OUTPUT
% none but saves many files
% -------------------------------------------------------------------------
% EXAMPLE
% % Double stripe analysis on Akrivi's PC
% basicdir='C:\Users\superuser\Example5';
% PerformanceAssessment_ShearWall(basicdir,'double')
% % Single stripe analysis on Akrivi's PC
% basicdir='C:\Users\superuser\Example5';
% PerformanceAssessment_ShearWall(basicdir,'single')
% % Single stripe analysis on Akrivi's PC with smaller font & marker sizes
% basicdir='C:\Users\superuser\Example5';
% PerformanceAssessment_ShearWall(basicdir,'single',16,16,8)

% _________________________________________________________________________
% Created 08/07/2019 by A. Chatzidaki & D. Vamvatsikos



if nargin<5, MarkerSize=10; end
if nargin<4, FigureFontSize=18; end
if nargin<3, LabelSize=22; end
if nargin<2, error('Need at least two arguments'); end



% ****************** setting the parameters ******************

%Define the basic directory where the results and the files are stored
%basicdir='C:\Users\superuser\Desktop\Example5_V5\Example5_V5';

% change the '\' to '/' (if any exists)
basicdir=regexprep(basicdir,'\','/');
cd(basicdir)


%AnalysisType='double';
% For double stripe analysis, the second stripe is at IMdoubleFactor*
% IMlevel level, where IM level is the IM of the first stripe
IMdoubleFactor=1.10;

% Change the paths where the Results will be stored 
% ResultsPath = path where the figures and the results will be stored
ResultsPath=[basicdir,'/Results/PerformanceAssessmentResults'];
% create the ResultsPath directory if it does not already exist
if ~exist(ResultsPath, 'dir'); mkdir(ResultsPath);  end

% Load the .sida.mat file with the IDA results
% IDAresultsPath = path where IDA results are stored
IDAresultsPath=[basicdir,'/IDAresults'];
% IDAFileName = Name of IDA results file
IDAFileName='ShearWallExample5.sida.mat';
% load the file
load(fullfile(IDAresultsPath,IDAFileName));

% Given cracking of concrete, it's better to use Sa(Teff) as the IM
% instead of Sa(T1). IDAs should be converted from Sa(T1) to Sa(Teff). 
% The Teff is stored in a .txt file found in the TeffPath path.
% Define the path where Teff.txt is stored
TeffPath=basicdir;
% Define the name of the file 
TeffFname='Teff.txt';
% Read the file 
Teff=dlmread(fullfile(TeffPath,TeffFname));

% define the seismic hazard to be used. Default is San Jose, California
SeismicHazardName='SanJose.hz.mat';

% Define the path where the records are stored(UseD to convert IDA results
% from Sa(T1) to Sa(Teff))
recorddir=[basicdir,'/RunIDA/bundle_runIDA/osees/records/'];




% define the limit of each PO

% maximum roof drift ratio (theta max) limit
IDRlimit=0.02; 
% column DCR limit
wallDCRlimit=1.0; 
% Define column plastic rotation capacity. We check the PO at 10%/50 ->
% Life Safety PO -> capacity is 0.017 (see book's page 530)
WallDCRcapacity=0.017;


% Define confidence level for each PO 
% confidence level for theta max limit
confIDR=0.70;
% confidence level for wall DCR
confDCRcolumns=0.85;

if or(strcmp(AnalysisType,'single')==1,strcmp(AnalysisType,'double')==1)
	% define the additional uncertainty to be employed for the demand capacity
	% factored design (DCFD)
	betacapacity=0.20;
	% Define the records whose results will be used for the stripe analysis
	% The following records were randomly selected.
	RecordsSaT1=[1,3,5,7,9,11,13,15,17,19,21,23,25,27,29,31,33]; % 17 records
    % records of the second stripe
	RecordsSaT1Double=RecordsSaT1;% the same records used for both stripes
    % Create a folder to store the results      
    ResultsPath=[ResultsPath,'/',AnalysisType,'stripe'];
    if ~exist(ResultsPath, 'dir'); mkdir(ResultsPath);  end

	MAFCapacityWallDCR=-log(1-0.10)/50;

elseif strcmp(AnalysisType,'MAFestimation')==1
	% An approximately equivalent dispersion in intensity measure (IM) terms of
	% 0.25 is instead adopted for the mean annual frequency (MAF) format
	bepist=0.25;
	
	% set the MAF limit for the Performance Objectives to be checked. We have
	% two limits: one of 30% probability of exceedance in 50ys and the other of
	% 1% probability of exceedance in 50 yrs 
	% Following a poissonian assumption, the Mean Annual Frequency (lambda) of
	% exceeding a PO is given by:
	% P = 1-exp(-lambda*t) -> lambda = -ln(1-P)/t.
    % where P is the probability of exceeding the PO in t years 
	% Be careful since in matlab ln is calculated using log function
	MAFCapacityIDR=-log(1-0.10)/50; 
	MAFCapacityWallDCR=-log(1-0.10)/50;  
	
	% create inside the ResultsPath a folder where the IDA results will be
	% stored, named MAFestimation
    if ~exist([ResultsPath,'/MAFestimation'], 'dir'); mkdir(ResultsPath,'/MAFestimation');  end

end


% Input Sadesign from Code Design Spectrum for the effective period of the
% building
Sa_Teff=0.539255091103966;


% ------- IDA process parameters (start) --------------------
% this is the drift value used to truncate the IDAs. As you can see in the
% opensees file, we employed relaxed material properties to help the 
% dynamic analysis converge. In order not to artificially increase the 
% strength of the wall, we apply a cut-off limit to the IDA results at 
% about 0.035 drift 
truncate_dvalue=0.035;

% Please note the output "tsv" parameter of
% the postIDA_simpleprocessIDA function. This provides the intensity
% measure values at which each IDA is truncated and is used in
% the post-processing of all limit states to set the IM limit
% at which IDAs should be truncated 

relaxNaNs=1; % Allow removal of "single" intermediate NaNs!
% line styles to use for plotting
linestyles={'k-','k-','k-','k-'};
% drift above which all drift values are considered infinite (collapse occurs)
% helps with fitting! Try setting it to 100 to see the difference with some badly
% behaved sets!
infdr=0.4;
% fractiles
qs=[0.16,0.50,0.84]';
% ------- IDA process parameters (end) --------------------



% define the colors for plotting  points on IDAs
% first stripe: light grey
Stripe1Color=[0.68,0.67,0.67];
% second stripe: dark grey
Stripe2Color=[0.46,0.44,0.44];
% MAF estimation: light grey
PointColor=[0.68,0.67,0.67];


% FigNumbering = figure numbering starts from this value. Finds active
% figures and starts plotting in the next figure. This is done to avoid
% plotting one figure on the top of the other. If you want to start plotting
% from a certain figure number you should change the FigNumbering
h =  findobj('type','figure');
FigNumbering = length(h)+1;

% ****************** setting the parameters (end) ******************

% load the seismic hazard data
hz=load(SeismicHazardName);

%%

% Post-process IDA curves to convert the Sa(T1) to Sa(Teff). We multiply
% the svalues of the IDA with the ratio of the Sa(Teff)/Sa(T1), where
% Sa(Ti) is estimated from the spectrum of the record for the given Ti
Nrecs=length(thfilename);
T1=T_periods(1,1);
svaluesT1=svalues;
clear svalues
Nrecs=length(thfilename);
svalues=cell(1,Nrecs);
Sa=cell(1,Nrecs);
for i=1:Nrecs
	[recinfo]=get_recinfo4_2([recorddir,thfilename{i}],'silva',0.05,[T1, Teff],1);
	% Note that here we add an extra check to make sure that everything
	% works fine. In the record_saxy output from IDA we get the value 
	% of the Sa for the periods of the building. 
	if ~(recinfo.sa(1,1)==record_saxy(i,1)); error(['please check results of record ', num2str(i)]); end
	Sa{i}=recinfo.sa;
	svalues{i}=svaluesT1{i}*Sa{i}(1,2)/Sa{i}(1,1);
end
   
% find the scale factor of the hazard curve. We want to scale the Hazard
% curve to get for 10%/50yr (which is equivalent to a MAF of 0.0021) the Sa
% value of the EC8 design spectrum.
[sf]=findSfHazardCurve(Teff,0.0021,Sa_Teff,hz,'interplinear',1,LabelSize,...
    FigureFontSize,MarkerSize);



%% MAX IDR
disp(['PO: IDR of ',num2str(IDRlimit)])
% define the IM level of the 1st stripe: the IM of the first stripe 
% corresponds to the MAFCapacityIDR estimated by the hazard curve
IMlevel=Sa_Teff;

% get the IDA results (IDRval) for the 1st stripe. Note that we keep the
% tsv values where individual IDAs are truncated to force IDAs
% of the other EDPs* to truncate at the same IM levels (*Engineering Demand
% Parameters)
Nvals={IMlevel,IDRlimit};
[cv,cvsa,sa_range,IDRval,qdr,savalIDR,dr_range,qSa,lowest_C,tdv,tsv,tdvaluesIDR,tsvaluesIDR]=...
	postIDA_simpleprocessIDA(svalues,dvalues,Nvals,qs,0.8,[],truncate_dvalue,relaxNaNs);
plot_cv_IDAs(tsvaluesIDR,tdvaluesIDR,cv,lowest_C,[0,0.08],FigNumbering,linestyles,'',0,1,1.5)
set(gcf,'color','white'); box on; set(gca,'fontsize',FigureFontSize); grid on
ylabel({'spectral acceleration';'S_a(T_{eff},5%) (g)'},'fontsize',LabelSize);
xlabel('maximum interstory drift ratio, \theta^{ }_{max}','fontsize',LabelSize);


if strcmp(AnalysisType,'single')==1;
	% plot the results of the 1st stripe for the selected records on the top of
	% IDA curves
	hold on
	plot(IDRval(RecordsSaT1),IMlevel*ones(1,length(RecordsSaT1)),'o','MarkerFaceColor',Stripe1Color,'MarkerEdgeColor','k','MarkerSize',MarkerSize); 
	saveas(gca,fullfile(ResultsPath,['IDR_',AnalysisType,'_stripe_IDAs.fig']));

	% define the EDP values of the first stripe
	EDPval={IDRval(RecordsSaT1)};

	[ IDRResults.FDrpo,IDRResults.FC]=CheckPOs_StripeAnalysis( AnalysisType,IDRlimit,EDPval,IMlevel,confIDR,betacapacity,sf,Teff,hz,2,[]);
	display(IDRResults)
	saveas(gca,fullfile(ResultsPath,['IDR_',AnalysisType,'_stripe_HC.fig']));
	save(fullfile(ResultsPath,['IDR_',AnalysisType,'_stripe_Results.mat']),'IDRResults')
elseif strcmp(AnalysisType,'double')==1;
	% plot the results of the 1st stripe for the selected records on the top of
	% IDA curves
	hold on
	plot(IDRval(RecordsSaT1),IMlevel*ones(1,length(RecordsSaT1)),'o','MarkerFaceColor',Stripe1Color,'MarkerEdgeColor','k','MarkerSize',MarkerSize);
	% define the records of the 2nd stripe, the same records were used for both
	% stripes
	RecordsSaT1Double=RecordsSaT1;
	% define the IM level of the 2nd stripe
	SaDouble=IMdoubleFactor*IMlevel;
	% get IDA results for the 2nd stripe
	NvalsDouble={SaDouble,[]};
	[~,~,~,IDRval2,~,~,~,~,~,~,~,~,~]=...
		postIDA_simpleprocessIDA(svalues,dvalues,NvalsDouble,qs,infdr,[],tsv,relaxNaNs);
	hold on
	% plot the second stripe on the top of IDA results
	plot(IDRval2(RecordsSaT1Double),SaDouble*ones(1,length(RecordsSaT1Double)),'o','MarkerFaceColor',Stripe2Color,'MarkerEdgeColor','k','MarkerSize',MarkerSize)
	saveas(gca,fullfile(ResultsPath,['IDR_',AnalysisType,'_stripe_IDAs.fig']));
	% define the IM levels of the two stripes
	IMlevel2=[IMlevel,SaDouble];
	% define the cell with the EDP values of each stripe
	EDPval={IDRval(RecordsSaT1);IDRval2(RecordsSaT1Double)};
	[ IDRResults.FDrpoIDRDouble,IDRResults.FCrIDRDouble ]=CheckPOs_StripeAnalysis( AnalysisType,IDRlimit,EDPval,IMlevel2,confIDR,betacapacity,sf,Teff,hz,2,[]);
	display(IDRResults)
	saveas(gca,fullfile(ResultsPath,['IDR_',AnalysisType,'_stripe_HC.fig']));
	save(fullfile(ResultsPath,['IDR_',AnalysisType,'_stripe_Results.mat']),'IDRResults')
elseif strcmp(AnalysisType,'MAFestimation')==1;
	% plot IDA results for drift = drift lim
	hold on
	plot(IDRlimit*ones(1,Nrecs),savalIDR,'o','MarkerFaceColor',PointColor,'MarkerEdgeColor','k','MarkerSize',MarkerSize); 

	% calculate the dispersion (i.e. log standard deviation of the data)
	betaIDR=std(log(sort(savalIDR)));
	% calculate the median IM
	medianIDR=median((sort(savalIDR)));
	[IDRMAF.lambdaXIDR,IDRMAF.lambda50IDR,IDRMAF.lambdamIDR]=CheckMAF(confIDR,medianIDR,betaIDR,Teff,1,sf,MAFCapacityIDR,bepist);
% 	display(IDRMAF)
% 	display(MAFCapacityIDR)
	saveas(gca,fullfile([ResultsPath,'/MAFestimation'],['IDR_',AnalysisType,'_IDAs.fig']));
	saveas(gca,fullfile([ResultsPath,'/MAFestimation'],['IDR_',AnalysisType,'_IDAs.png']));
	save(fullfile([ResultsPath,'/MAFestimation'],['IDR_',AnalysisType,'_Results.mat']),'IDRMAF')
end




%% Plastic rotation at the base of the wall
disp(['PO: Plastic rotation at the base of the wall, DCR = ',num2str(wallDCRlimit)])

% We will employ a trick to estimate the rotation at the base of the wall.
% It is computed as theta,inelastic using the following 
% equation:
% theta,inelastic = theta,roof,total - theta,roof,elastic 
% where theta,roof,elastic is estimated from the initial slope of the
% pushover curve, theta,roof,total is the roof drift ratio of the pushover
% curve and theta,inelastic is the difference between the other two

% These values are stored in ThetaRoof.mat file, located in the
% folder of IDA results, but it can be easily calculated 
load(fullfile(IDAresultsPath,'TheteRoofDB.mat'));

% change the format of the rdr results and convert them to column DCR by
% interpolating the total roof drift ratio with the inelastic roof drift
% ratio. We directly divide it by the plastic rotation capacity of the
% wall to get the DCR
rdvalues=cell(1,Nrecs);
RDRinelastic=cell(1,Nrecs);
for i=1:Nrecs
	rdvalues{i}=anlsout(i).maxrdrx';
	rdvalues{i}(1,1)=0;
	RDRinelastic{i}=interp1(ThetaRoof.total,abs(ThetaRoof.inelastic),rdvalues{i})/WallDCRcapacity;
end

Nvals={IMlevel,wallDCRlimit};

% note the "tsv" defined herein in the postIDA_simpleprocessIDA function,
% which is the output of the corresponding function of post-processing
% the maximum interstory drift ratio. What we do is to get the IM values
% where each IDA is truncated (because of the truncation limit of the
% maximum interstory drift ratio) and force the IDAs for a different EDP to
% be truncated at the same IM values.
[cvWallDCR,cvsa,sa_rangeWallDCR,DCRwall1,qdr,savalWallDCR,dr_range,qSa,lowest_CWallDCR,tdv,~,tdvaluesWallDCR,tsvaluesWallDCR]=...
	postIDA_simpleprocessIDA(svalues,RDRinelastic,Nvals,qs,[],[],tsv,relaxNaNs);
plot_cv_IDAs(tsvaluesWallDCR,tdvaluesWallDCR,cvWallDCR,lowest_CWallDCR,[-0.003,2.0],FigNumbering+6,linestyles,'',0,1,1.5)
set(gcf,'color','white'); box on; set(gca,'fontsize',FigureFontSize); grid on
ylabel({'spectral acceleration';'S_a(T_{eff},5%) (g)'},'fontsize',LabelSize);
xlabel('wall DCR','fontsize',LabelSize);


if strcmp(AnalysisType,'single')==1;
	IMlevel=Sa_Teff;
	hold on;
	plot(DCRwall1(RecordsSaT1),IMlevel*ones(1,length(RecordsSaT1)),'o','MarkerFaceColor',Stripe1Color,'MarkerEdgeColor','k','MarkerSize',MarkerSize);
	saveas(gca,fullfile(ResultsPath,['WallDCR_',AnalysisType,'_stripe_IDAs.fig']));	
	EDPval={DCRwall1(RecordsSaT1)};
	[ WallDCR.FDrpoDuctSingle,WallDCR.FCrDuctSingle ]=CheckPOs_StripeAnalysis( AnalysisType,wallDCRlimit,EDPval,IMlevel,confDCRcolumns,betacapacity,sf,Teff,hz,1,[]);
	display(WallDCR)
	saveas(gca,fullfile(ResultsPath,['WallDCR_',AnalysisType,'_stripe_HC.fig']));	
	save(fullfile(ResultsPath,['WallDCR_',AnalysisType,'_stripe_Results.mat']),'WallDCR');
elseif strcmp(AnalysisType,'double')==1;
	hold on;
	plot(DCRwall1(RecordsSaT1Double),IMlevel*ones(1,length(RecordsSaT1Double)),'o','MarkerFaceColor',Stripe1Color,'MarkerEdgeColor','k','MarkerSize',MarkerSize);
	saveas(gca,fullfile(ResultsPath,['WallDCR_',AnalysisType,'_stripe_IDAs.fig']));	
	SaDouble=IMdoubleFactor*IMlevel;
	NvalsDouble={SaDouble,[]};
	[~,~,~,DCRwall2,~,~,~,~,~,~,~,~,~]=postIDA_simpleprocessIDA(svalues,RDRinelastic,NvalsDouble,qs,[],[],tsv,relaxNaNs);
	hold on
	plot(DCRwall2(RecordsSaT1Double),SaDouble*ones(1,length(RecordsSaT1Double)),'o','MarkerFaceColor',Stripe2Color,'MarkerEdgeColor','k','MarkerSize',MarkerSize)
	saveas(gca,fullfile(ResultsPath,['WallDCR_',AnalysisType,'_stripe_IDAs.fig']));
	IMlevel2=[IMlevel,SaDouble];
	EDPval={DCRwall1(RecordsSaT1);DCRwall2(RecordsSaT1Double)};
	[ WallDCR.FDrpoDuctDouble ,WallDCR.FCrDuctDouble]=CheckPOs_StripeAnalysis( AnalysisType,wallDCRlimit,EDPval,IMlevel2,confDCRcolumns,betacapacity,sf,Teff,hz,1,[]);
	display(WallDCR)
	saveas(gca,fullfile(ResultsPath,['WallDCR_',AnalysisType,'_stripe_HC.fig']));
	save(fullfile(ResultsPath,['WallDCR_',AnalysisType,'_stripe_Results.mat']),'WallDCR')
elseif strcmp(AnalysisType,'MAFestimation')==1;
	% plot IDA results for drift = drift lim
	hold on
	plot(wallDCRlimit*ones(1,Nrecs),savalWallDCR,'o','MarkerFaceColor',PointColor,'MarkerEdgeColor','k','MarkerSize',MarkerSize);
	betaRDR=std(log(sort(savalWallDCR)));
	medianRDR=median((sort(savalWallDCR)));
	[WallDCR.lambdaXIDR,WallDCR.lambda50IDR,WallDCR.lambdamIDR]=CheckMAF(confDCRcolumns,medianRDR,betaRDR,Teff,1,sf,MAFCapacityWallDCR,bepist);
% 	display(WallDCR)
% 	display(MAFCapacityWallDCR)
	saveas(gca,fullfile([ResultsPath,'/MAFestimation'],['WallDCR_',AnalysisType,'_IDAs.fig']));
	saveas(gca,fullfile([ResultsPath,'/MAFestimation'],['WallDCR_',AnalysisType,'_IDAs.png']));
	save(fullfile([ResultsPath,'/MAFestimation'],['WallDCR_',AnalysisType,'_Results.mat']),'WallDCR');
end



