function [y,flag,t]=x2y(x,cv)
% function [y,flag,t]=x2y(x,cv)
%-----------------------------------------------------------------------
% INPUT
% cv : Natural Spline curve produced thru "cscvn" or similar
%      it is essentialy a piecewise polynomial in pp-form, that
%      uses a parameter t to produce (x,y) values.
%      i.e. cv(t)=(x(t),y(t)) ( R-->R^2 function )
%      what this routine does is take x value, find corresponding t
%      and then produce y value. Since "cv" is 2D curve, you may get
%      multiple y's
% x  : Value of x whose corresponding y you wish
% OUTPUT
% y  : Value(s) of y corresponding to x. Could be a whole vector!
% flag : If it is 1 then the specified x is not producing an acceptable
%        value of "t", i.e the routine will only interpolate(flag=0), but
%        not extrapolate (flag=1). If such an "x" is required, an obsene 
%        value of y=10 will be returned and fag is raised to 1.
% t    : an optional output, just for use by x2dxy.m
%        It contains the corresponding t-parameter values.
%-----------------------------------------------------------------------
% NOTE : if you wish a "y2x" operation (i.e supply y and get x),
% all you have to do is fit cv2=cscvn(points), where oints contains
% the (x,y) points as (y,x) now. It is the same curve as "cv", just the
% y's precede the x's. Now it is easy to see that 
% x=x2y(y,cv2), y=x2y(x,cv), 
% x=x2y(x2y(x,cv),cv2)  ( just playing around!)
% KNOWN PROBLEM: It is very unusual, but due to round-off problems
% this may return 2 y's for one x, and they will be very close,
% almost identical and identical to a break point. This is due to 
% computer imprecision, can't do anything about it!
% For the same reason sometimes it will not recognize the last curve point
% as a root, so we need to specify a tolerance to include it too.
% thankfully the tolerance can be very low (10^-14 is used here).
% CAUTION : 
% when it is used inversly, i.e x=drift,y=Sa,  sometimes the same x  has 
% multiple corresponding y's (Sa's). Then, as in the output the y's
% come in the order they were found, "y" is ordered in increasing
% parameter "t", which almost always means increasing Sa. This is 
% useful to interpret the output of x2dxy.m that uses x2y.m . See 
% x2dxy.m for details.

% (Last Updated: May 31st, 1999)
% Updated Feb 21, 2005 : Fixed the returning of tvalues that are slightly
%                        higher than max(breaks). These are truncated down
%                        to max(breaks). The difference is usually only 1e-16!
% Copyright (c) 1999 by D.Vamvatsikos


% tolerance measure. Solely used to include the points close to 
% the knots, that usually may be missed due to round off errors
tol=1E-14;


flag=0; t=[];
[brk,C,n,k]=ppbrk(cv);   % get breaks, polynomial coefficients and # of breaks 
% convert x to polynomial of order k-1 with x as constant term. 
% if you don't do that, subtracting "x" from C(i*2-1,:) will take x out of
% all coefficients, not just the constant!
xp=zeros(1,k) ; xp(k)=x;
for i=1:n
  %// remember to sort roots found (if 2 found at the same interval
  %// nothing guarantees they are going to be ordered. This helps
  %// later when we wish multiple y-outputs to come in order of
  %// increasing "t" parameter value.
  r=sort(roots(C(i*2-1,:)-xp))+brk(i);  % find roots of each polynomial piece
  % now choose only those roots that exist within interval of that pp.
  % the local coords are [0,brk(i+1)-brk(i)] --> domain of def. of i-part.
  % NOT [brk(i),brk(i+1)], but if we add brk(i) to the roots we return
  % to global coords. The very last point (brk(n)) is not
  % accepted as a root. Well, not a big problem! Also check the transpose 
  % needed for r(find...). That is because if 2 or three of r's components
  % pass the test then you will be trying to concatenate t (row vector)
  % with a column vector (r(find..)), so transpose the column vector!
  t=[t,r(find(r>=brk(i) & r<brk(i+1)+tol & imag(r)==0))'];
end
  % now correct, and accept last point of cv as possible root.
  % allow some tolerance, as sometimes the round-off error wil
  % provide a slightly higher root!

  %// I think this has already been taken care of above!
 % t=[t,r(find(r>=brk(n+1) & r<brk(n+1)+tol &  imag(r)==0))]; 

if isempty(t)
  flag=1; % extrapolation danger. Raise flag.
  y=NaN;
else
  t=unique(t);  %// get rid of repetitions that "tol" might include
  % Make sure that any solutions found that have a higher tvalue than the 
  % maximum break should be converted to the max break. This is because of the
  % tolerance included above. These values usually only differ by 1e-16 from the
  % maximum break (max interpolated\allowable t-value) and they are caused by
  % inclusion of "tol" above!
  t(t>max(brk))=max(brk);
  xy=fnval(cv,t);  % now evaluate the "cv" curve at found "t"
  y=xy(2,:);  % separate the y's from xy points
% if any( x~=xy(1,:)), disp('Error in x2y.m'), end  % all these should be equal
end
