function [y,flag,t]=x2y_s(x,cv)
% function [y,flag,t]=x2y_s(x,cv)
% This function is similar to x2y() but features a faster computation of
% a vector of x's (not just a single value) by assuming a monotonic "cv"
% curve.
%-----------------------------------------------------------------------
% INPUT
% cv : Natural Spline curve produced thru "cscvn" or similar
%      it is essentialy a piecewise polynomial in pp-form, that
%      uses a parameter t to produce (x,y) values.
%      i.e. cv(t)=(x(t),y(t)) ( R-->R^2 function )
%      what this routine does is take x value, find corresponding t
%      and then produce y value. Since "cv" is 2D curve, you may get
%      multiple y's, but this routine will only return the one that
%      is closest to (0,0) along the curve.
% x  : A SORTED vector of x whose corresponding y you wish
% OUTPUT
% y  : Values of y corresponding to x. While x2y.m could produce more than
%      one values of y for a single x, this one assumes that 
%      "cv" is a monotonic curve, so it is 1-1. This should always be
%      the case with DPOs but sometimes the spline interpolation messes
%      this a bit (in uper tail). Even then usually only the "y" value 
%      that is closest (along the curve) to the origin (0,0) is used.
%      this routine captures exactly that value! (by the way that is
%      not always the min(y)!)
% flag : If it is 1 then the specified x is not producing an acceptable
%        value of "t", i.e the routine will only interpolate(flag=0), but
%        not extrapolate (flag=1). If such an "x" is required, an obsene 
%        value of y=10 will be returned and fag is raised to 1.
% t    : an optional output, just for use by x2dxy.m
%        It contains the corresponding t-parameter values.
%-----------------------------------------------------------------------
% 

% (Last Updated: August 27th, 1999)
% Copyright (c) 1999 by D.Vamvatsikos

% tolerance measure. Solely used to include the last point of cv
% as a possible root. See below.
tol=1E-14;

%// make "t" all NaN and same size as x.

flag=ones(size(x)); t=x*NaN;

[brk,C,n,k]=ppbrk(cv);   % get breaks, polynomial coefficients and # of breaks 
% convert x to polynomial of order k-1 with x as constant term. 
% if you don't do that, subtracting "x" from C(i*2-1,:) will take x out of
% all coefficients, not just the constant!
xp=zeros(1,k);
%// "i" counts roots, "j" counts x-values
i=1;j=1;

while (i<=n) & (j<=length(x))
 
  xp(k)=x(j);
  %// remember to sort roots found (if 2 found at the same interval
  %// nothing guarantees they are going to be ordered. This helps
  %// later when we wish multiple y-outputs to come in order of
  %// increasing "t" parameter value.

  r=sort(roots(C(i*2-1,:)-xp))+brk(i);  % find roots of each polynomial piece

  % now choose only those roots that exist within interval of that pp.
  % the local coords are [0,brk(i+1)-brk(i)] --> domain of def. of i-part.
  % NOT [brk(i),brk(i+1)], but if we add brk(i) to the roots we return
  % to global coords. The very last point (brk(n)) is not
  % accepted as a root. Well, not a big problem! Also check the min() 
  % needed for r(find...). That is because if 2 or three of r's components
  % pass the test then you will want to keep the lowest one only.

  temp=min(r(find(r>=brk(i) & r<brk(i+1)+tol & imag(r)==0)));

  if isempty(temp)
    % no root found in this interval, proceed to next one
    i=i+1;
  else 
    % root was found, proceed to next x-value
    t(j)=temp;
    flag(j)=0;
    j=j+1;
  end   
end

  
xy=fnval(cv,t);  % now evaluate the "cv" curve at found "t"
y=xy(2,:);  % separate the y's from xy points


