function [c_Sa,c_drift,c_flag]=truecap5(xy,cv,Kelastic,frac,Th_drift, ...
                                        criteria,debugflag)
%function [c_Sa,c_drift,c_flag]=truecap5(xy,cv,Kelastic,frac,Th_drift,...
%                                      criteria,debugflag)
%----------------------------------------------------------------------
% Provides a Capacity in Sa or drift terms for the given curve.
% The criterion is tangent slope.
% INPUT
% xy        : array Nx2 of real (Sa,drift) points, SORTED by
%             Sa. Remember, 1st column is IM, second is DM.
% cv        : A curve fitted  by fitDPO,cscvn2 or cscvn
%             of Sa GIVEN drift (y=Sa, x=drift) on the "xy" points
%             If empty or not given, then the routine fits through
%             fitDPO.m
% Kelastic  : the elastic stiffness to use. If not given, it is defined
%             by the second point of the DPO.
% frac      : frac=1/Pcap, it is the fraction of Kelastic to use to define
%             the capacity slope: targetE=frac*Kelastic.
%             default is 20%.
% {targetE  : dx/dy slope where capacity is assumed to be.
%             If not provided it is assumed to be 20% of the elastic 
%             stiffess (as defined by second DPO point).}
% Th_drift  : a threshold drift (usually 10%), which is the 
%             maximum collapse drift. If not provided, a default
%             10% is assumed
% criteria  : Denotes whether we use the standard definition of
%             capacity whether a more advanced robust version
%             If 0 or not given, then the standard is used.
%             If 1 then the robust version is used.
% debugflag : if 1 then the candidate slope points are printed.
% OUTPUT
% c_Sa      : scalar, value of Sa (g) @ capacity
% c_drift   : scalar, value of drift @ capacity
% c_flag    : flag denotes type of point designated as capacity.
%             values of 1,2,3 correspond to types a,b1,b2 points
% --------------------------------------------------------------------
% NOTE : I have recently disabled the plotting of curve + capacity 
% that this routine used to have. If you want it back, take off the "%". 
% UPDATED, Oct 4, 1999 : It uses cscvn2.m for better fits, plus it incorporates
%    the new definition of capacity, that makes more sense and is 
%    more robust.
% Updated, Feb 7, 2005 : Slight modification to use updated flipcscvn.
% Copyright (c) 1999 by D.Vamvatsikos

%subplot(1,2,2) 
%format compact

% Check Input arguments, enforce default options if neede
if nargin<1
   error('too few arguments')
end

if nargin<2
   cv=[]; % force a fit!
end

if nargin<3
   Kelastic=xy(2,1)/xy(2,2);
end

if nargin<4 
   frac=0.2;  %// default is 1/5 of elastic!     
end

if nargin<5
   Th_drift=0.1;
end

if nargin<6 
   criteria=0;  
end

if nargin<7, debugflag=0; end;  %//by default don't debug.

targetE=Kelastic*frac;

if isempty(cv) 
  cv=fitDPO(xy(:,1),xy(:,2)); %//x=drift, y=sa 
  warning('Refitting points, no fit provided!')
end

% careful, now cvsa is a cell array due to changes in flipcscvn.
%
cvsa=flipcscvn({cv}); %//' same as cv but now x=sa,y=drift

%// Start collecting all the candidate points for capacity.
%// These include 
%//  a)All points on softening curve with a slope of targetE 
%//  b1)The lowest in terms of Sa point that has a drift of Th_drift.
%//    OR
%//  b2)The last computed point of the DPO, provided it has lower
%//    drift AND Sa than point (b1) above (if point b1 exists)
%// you can either have (a&b1) or (a&b2), as b1 and b2 are options 
%// for the terminal cut-off point that are mutually exclusive.
%keyboard

%// Find all type (a) points (slope points)
    [x,y,ddydx,flag]=dxy2xy(1/targetE,cvsa{1});
    %// Find a cutoff point type (b1) or (b2)
    [x2,flag2]=x2y(Th_drift,cv);
    if flag2==1 
       %// no type (b1) point, so type (b2) is going to be our cut-off
       %// as it is certainly below Th_drift in drift terms! 
       x2=xy(end,1);y2=xy(end,2);
    else
       %// select the minimum (in terms of Sa) type (b1) point!   
       x2=min(x2);y2=Th_drift;
       %// certainly the type (b2) point is far above in Sa terms, 
       %// so there is no need to consider it.
    end

    %// just for debugging purposes.
    if debugflag
      fprintf('Candidate slope points \n')
      [x(ddydx>0);y(ddydx>0)]'
    end  

    % If flag is raised, check to see whether it was due to
    % very low or very high slope. If very low targetE( almost 0)
    % then assign last point as capacity and set flag to 0. Otherwise
    % if targetE > Elastic slope, then leave flag raised and set
    % x,y to 0. Obviously an invalid targetE value was supplied 
    % this check could be done earlier, but since hardening may allow
    % slopes>Elastic, we leave this be as an interesting exercise!

    %// remember, (x,y) are sorted towards higher Sa,
    %// exclude all (x,y) that are higher (in Sa) than the cutoff
    %// point (x2,y2) plus they must reside on softening slope
    indx=find(x<x2 & ddydx>0);
    x=x(indx); y=y(indx);
    %keyboard
    if (flag==1)|(isempty(x)) 
       %// set type b1 or b2 point to be capacity.
       %// if no type (a) point qualifies. 
       c_drift=y2;
       c_Sa=x2;
       if flag2==1 c_flag=3; else c_flag=2; end
    else
     %// Type (a) points exist that are earlier than the cutt-off point.
     if criteria==1
       %// use robust definition of capacity
       x=[x,x2];y=[y,y2];
       i=1;
       while 1
         % //approximate tangent slopes :dydx=diff(y)./diff(x);
         % //but we want slopes of each i-point with its following ones.
         % //i.e. secant slopes with origin the given i-point 
         indx=(i+1):length(x);
         %// slope here is stiffness, i.e. dSa/dtheta --> dx/dy !! 
         slopes=(x(indx)-x(i))./(y(indx)-y(i));
         %//rise is the increase in Sa  
         rises=x(indx)-x(i);
         if any((slopes>targetE)|(rises>frac*x(i)))&(i<length(x))
            %// there exists a significant hardening later on. 
            %// advance to next point 
            %// if i==length(x) though, there is no next point to
            %// go to so this is our capacity 
            i=i+1;
         else
            %//no significant hardening, we found our capacity.
            if i==length(x)
              %// cutoff point assigned to capacity.
              if flag2==1 c_flag=3; else c_flag=2; end
            else
              c_flag=1;
            end
            c_Sa=x(i);
            c_drift=y(i);
            break;
         end
       end
     else
       %// the standard definition of capacity. Last slope point is it!
       c_Sa=x(end);
       c_drift=y(end);
       c_flag=1;
     end
    end  
   

% generate nice plots in (drift,SA) axes (visually better!)
%cv2=cscvn([xy(:,2),xy(:,1)]'); %//'
%fnplt(cv2,'g',1); hold on; 
%plot(y,x,'yo')  %// plot all points that have targetE slope.
%plot(drift,Sa,'rs')  %// plot red the selected as capacity point
%plot(xy(:,2),xy(:,1),'ko') %// plot the original curve points.

%// set axis to make a nice plot, similar to driverPL plots!
%axis([0,1.05*max(xy(:,2)),0,1.05*max(xy(:,1))])
