function [svalues,dvalues,anlsout,T_periods,runs,record_saxy,...
                                     record_pgaxy]=runIDA_ND(anls,trace,useSNAP);
% [svalues,dvalues,anlsout,T_periods,runs,record_saxy,...
%                                     record_pgaxy]=runIDA_ND(anls,trace,useSNAP);
% Runs IDAs for a suite of records
%_________________________________________________________________________
% INPUT 
%   anls : structure containing record filenames, directories etc
%   trace: structure containing the parameters needed for the tracing
%          algorithm
%   useSNAP: if 1 then use snap fortran program to get SA info from
%          records. Default is 0 which uses my own Matlab functions.
% OUTPUT
%   svalues,dvalues: cell arrays 1x(maxrec) containing IM and EDP values
%   anlsout : cell array containing extra EDPs for each IM level
%   T_periods: the first N (where N is determined by the Opensees file)
%              periods of the structure
%   runs : vector 1x(maxrec) containing the number of actual runs
%   record_saxy : Each record's (and each component's) Sa(Ti,x%) values
%                 for each period of T_periods
%   record_pgaxy: Each record's (and component's) PGA.
% EXAMPLE
%   See runIDA_G7bridge.m 
%_________________________________________________________________________
% NOTES
% 1. It now has the ability to save between IDAs and recover automatically.
% 2. It searches for the savefilename and the temporary savefilename
%    in the analysis directory, not in the current one (as old versions did)
% 3. This can handle 1,2 or 3 components of motion. It allows you to 
%    choose them separately, and select which one will be directly scaled, the 
%    rest being scaled relative to it by some fixed factor (usually 1). If 
%    you have a 2D structure, a maximum of two components should be used: X,Y. 
%    The opensees file should be setup so that "Y" is treated as the vertical 
%    (not Z). This is natural for 2D coordinates of course!!!
% 4. There is no need to capture the outputs, as they are also saved in 
%    the anls.savefilename provided.
%
% Created Jan 21, 2005 by D.Vamvatsikos
% Updated Feb 03, 2005: minor corrections for clarity and speed.
% Updated Apr 25, 2005: Added support for anls.comp being a vector of 3
%                       rather than a single number, also now the
%                       savefilename and tmpsavefilename are considered
%                       relative to the anlsdir directory. This fixes an
%                       error where they were not detected and possibly
%                       in danger of being overwritten.
% Updated Jun 10, 2005: The names of the record-files are now saved
%                       in the final save-file as variable "thfilename"
% Updated Oct 05, 2005: Added the option anls.exefile, which defines what 
%                       openSees executable to use (and where it resides).
% Updated Oct 19, 2005: Fixed some minor error when printing the names of
%                       the records used for each run. The correct ones
%                       were used but wrong names appeared on screen.
% Updated Oct 19, 2005: Now it calls get_recinfo3 instead of get_recinfo2
%                       and it receives Pseudo-Sa information as well. The
%                       newer routine is also suitable for PEER NGA
%                       records
% Updated 10/May/2006 : Now it also saves the PSA information in the
%                       savefile and tmpsavefile. A major bug has been
%                       fixed where regardless of anls.comp being e.g.
%                       [1,3,1] the 1st component would be called
%                       dominant (i.e. the one to which Sa-values
%                       correspond to) rather than the second. 
% Updated 26/Nov/2011:  Added the useSNAP option to force using of SNAP as usually done. 
%                       The default now is employing my Matlab routines
%                       (they are faster!)

if nargin<3, useSNAP=0; end

% we are going to use this value a lot!
ncomp=sum(anls.comp~=0);
if ~all(ismember(anls.comp,[3,1,0])), error('Only 3/1/0 allowed for xyz components'); end
if all(anls.comp==0), error('need at least one non-zero component'); end
if ~(sum(anls.comp==3)==1), error('only one dominant (3) component may be used'); end
if length(anls.comp)>3
   error('only 3 components allowed')
elseif length(anls.comp)<3
   % fill with zeros up to length=3
   anls.comp=[anls.comp(1:end),zeros(1,3-length(anls.comp))];
end
if useSNAP
   snapdir=get_snapdir;
end
tmpsavefilename=fullfile(anls.anlsdir,['tempsave_' anls.savefilename]);
anls.savefilename=fullfile(anls.anlsdir,anls.savefilename);
if mod(length(anls.thfilename),ncomp)~=0, error('Incorrect number of records'); end
maxrec=floor(length(anls.thfilename)/ncomp);

[notfoundflag]=check_record_fnames(anls.recorddir,anls.thfilename);
if notfoundflag, error('Record(s) not found'); end
% Check existence of savefilename & tmpsavefilename
file_overwrite_check(anls.savefilename);
if exist(tmpsavefilename,'file')
   fprintf('Recovering from %s\n',tmpsavefilename)
   load(tmpsavefilename);
   istart=length(svalues)+1;
   fprintf('Recovered %d records, %d total\n\n',istart-1,maxrec);
else
   istart=1;
end

% Preprocessing: Get the periods of the structure
[T_periods] = getperiods_osees(anls.anlsdir,anls.exefile,anls.infile,anls.outdir);

for i=istart:maxrec
      t0=clock;
      % Preprocessing: Get the Sa(Ti,5%) values for the 2*(i-1)+1 record and the next, 
      % i.e. for all the records used for x,y,z direction.
      for j=1:ncomp 
         thname{j}=[anls.recorddir '/' anls.thfilename{ncomp*(i-1)+j}];
         if useSNAP
            snap_clear(thname{j});
            [rinfo,Eflag]=get_recinfo3(thname{j},...
               anls.rectype,anls.def_zeta,T_periods',snapdir);
         else
            [rinfo,Eflag]=get_recinfo4(thname{j},...
               anls.rectype,anls.def_zeta,T_periods');
         end
         recinfo(ncomp*(i-1)+j)=rinfo;
         % collect all the sa's for all periods in Tperiods in one variable
         record_saxy(ncomp*(i-1)+j,:)=rinfo.sa;
         % similarly collect all the pga's
         record_pgaxy(ncomp*(i-1)+j,:)=rinfo.pga;  
         record_psaxy(ncomp*(i-1)+j,:)=rinfo.psa;  
      end
      fprintf('\n*Starting IDA tracing with records\n')
      rc=1;
      cletters={'X','Y','Z'};
      for ii=find(anls.comp~=0)
         fprintf('     %s: %s\n',cletters{ii},anls.thfilename{(i-1)*ncomp+rc})
         rc=rc+1;
      end
      % the component that was designated as "3" in anls.comp will be used
      % to calculate the Sa(Ti) for scaling. So idomc becomes a number: 1,2
      % or 3. 
      idomc=find(anls.comp==3);
      fprintf('*Scaling by Sa(T%d),T%d=%gs as calculated for %s-component\n',...
              anls.iscaleT,anls.iscaleT,T_periods(anls.iscaleT),...
              cletters{idomc});
      % put here all the extra input you want for the analysis routine, 
      % mostly specifying which opensees file to run!
      % CAREFUL, do not use fullfile here as the separator will become "\" for
      % windows machines and it will force TCL into an error.
      % CAREFUL: Supply recinfo for the dominant component, idomc! Get the
      % proper Sa value (for T1 or T2 or ... according to anls.iscaleT) for
      % this component.
      blackboxinput={thname,anls.comp,recinfo(ncomp*(i-1)+idomc).sa(anls.iscaleT),...
                     ones(1,ncomp-1),i,maxrec,...
                     anls.anlsdir,anls.exefile,anls.infile,anls.outdir,...
                     anls.blackbox_postproc,anls.postproc_input};
      % Run traceIDA on the anlsproc_osees!
      [svalues{i},dvalues{i},runs(i),NCSa(i),transout{i}]=...
         traceIDA(trace.maxrpr,trace.iniSa,trace.incSa,trace.Th_drift,trace.Th_Sa,...
                  trace.mode,trace.hf,trace.eSa,trace.edr,...
                  anls.blackbox_anlsproc,blackboxinput{:});   
      fprintf('>>>>>> Record %d/%d : Total time %0.4g hrs\n',i,maxrec,etime(clock,t0)/3600)
      save(tmpsavefilename,'svalues','dvalues','transout','T_periods','runs',...
                           'record_saxy','record_pgaxy','record_psaxy')
end

% transform everything from transout format to the more useful anlsout format.
%keyboard
[anlsout]=trans2ida_osees(transout,maxrec);
thfilename=anls.thfilename;

save(anls.savefilename,'svalues','dvalues','anlsout','T_periods','runs',...
                           'record_saxy','record_pgaxy','record_psaxy','thfilename')