function ShearWall_analysis(basicdir,analysistype,LabelSize,FigureFontSize)
% ShearWall_analysis(basicdir,analysistype,LabelSize,FigureFontSize)
%
% Using this file you can run pushover, dynamic or IDA analysis on the
% shear wall. You can also plot the wall section or the model, only 
% if you have already run pushover or dynamic analysis
% -------------------------------------------------------------------------
% INPUT
% basicdir:.    directory where ShearWall_analysis file is located
% analysistype: 'pushover'    run pushover analysis
%               'dynamic'     run dynamic analysis
%               'IDA'         run Incremental Dynamic Analysis (IDA)
%               'plotSection' plot the section of the shear wall 
%                             (available only if you have already run 
%                             pushover analysis or dynamic analysis)
%               'plotModel'   plot the model of the shear wall
%                             (available only if you have already run
%                             pushover or dynamic analysis)
%        (not that this parameter is not case-sensitive)
% LabelSize:      size of x and y axes labels of figures
% FigureFontSize: size of fonts in figures
% -------------------------------------------------------------------------
% OUTPUT
% none but saves many files
% -------------------------------------------------------------------------
% EXAMPLE
% % Run pushover analysis in Akrivi's PC
% ShearWall_analysis('C:\Users\superuser\Example5','pushover')
% % Plot wall section on Akrivi's PC (available only if you have run
% pushover or dynamic analysis)
% ShearWall_analysis('C:\Users\superuser\Example5','plotSection')
% % Run IDA analysis in Akrivi's PC
% ShearWall_analysis('C:\Users\superuser\Example5','IDA')
% _________________________________________________________________________
% Created 08/07/2019 by A. Chatzidaki & D. Vamvatsikos


if nargin<4, FigureFontSize=18; end
if nargin<3, LabelSize=22; end
if nargin<2, error('Need at least two arguments'); end


% ****************** setting the parameters ******************

% define the basic directory where the ShearWall_analysis.m file is
% located. 
% basicdir='C:\Users\ShearWall\Example5';


% change the "\" to "/" in the directory
basicdir=regexprep(basicdir,'\','/');

% Define the analysistype. Options:
%analysistype = 'pushover'; %  runs pushover analysis
%analysistype = 'dynamic'; %  runs dynamic analysis of a single record
%analysistype = 'IDA'; %  runs IDA analysis
%analysistype = 'plotSection'; %  plots the wall section
%analysistype = 'plotModel'; %  plots the model created in Opensees

% Define the folder where the .tcl files of the Shear wall are stored. By
% default they are in:
FilesPath=[basicdir,'/OpenseesModel/Model'];
cd(FilesPath)

% define the path where you want to store the results. The default path is 
% basicdir/Results
ResultsPath=[basicdir,'/Results'];
% Create this folder if it does not already exist
if ~exist(ResultsPath, 'dir'); mkdir(ResultsPath);  end


% Define the height of the building (used to convert the roof
% drift ratio to roof displacement), input in meters
BldgHeight= 29; %m


if strcmpi(analysistype,'IDA')==1
    % For IDA analysis, the following directories should be defined:

	% directory of the records 
	recorddir=[basicdir,'/RunIDA/bundle_runIDA/osees/records'];
	% change '\' slashes to '/' 
    recorddir=regexprep(recorddir,'\','/');
    
    % Define the path where the Osees files will be stored to run IDA
    % analysis (they will be copied automatically in this path)
	anlsdir  =[basicdir,'/RunIDA/bundle_runIDA/osees/ShearWall'];
	% Path where the runIDA_Shear_Wall function is stored
	IDAPTpath=[basicdir,'/RunIDA/bundle_runIDA'];
    
elseif strcmpi(analysistype,'plotSection')==1
    % define the story whose section you want to plot (default is story 1)
    Story=1;
end

% ****************** setting the parameters (end) ******************


if strcmpi(analysistype,'pushover')==1
	% Run pushover analysis and plot the results
	
	% write the .tcl file with the type of analysis 
	write_MatIDA_paramfile(FilesPath,'Mat_anlsfile.tcl',0,...
        'analysistype',analysistype)
	% Run pushover analysis. During the pushover analysis you can
	% see the deformed model. If you want to disable this
	% option, you can comment lines 345-349 in 
    % .../OpenseesModel/Model/ShearWall_model.tcl file
	!Opensees.exe ShearWall_model.tcl
	
	% read the roof drift and plot the pushover curve in terms of base
	% shear (kN) and roof displacement (m, =roof drift * BldgHeight)
	RoofDrift=dlmread(fullfile(FilesPath,'roofdrx.out'));figure; hold on
	plot(RoofDrift(:,2)*BldgHeight,RoofDrift(:,1),'k-','linewidth',2)
	xlabel('roof displacement (m)','fontsize',LabelSize)
	ylabel('base shear (kN)','fontsize',LabelSize)
	grid on; box on; set(gca,'fontsize',FigureFontSize)
	% store the pushover results figure in the ResultsPath/pushover
	% directory
    % create this directory if it doesnt exist
    PushoverPath=[ResultsPath,'/',analysistype];
	if ~exist(PushoverPath, 'dir'); mkdir(PushoverPath);  end
	% save the figures in .fig and .png format
	saveas(gca,fullfile(PushoverPath,'Pushover'))
	saveas(gca,fullfile(PushoverPath,'Pushover'),'png')
	
	
elseif strcmpi(analysistype,'dynamic')==1
	% run dynamic analysis and plot thetaroof - dt figure
	
	% write the .tcl file with the analysis type to be performed 
	write_MatIDA_paramfile(FilesPath,'Mat_anlsfile.tcl',0,...
        'analysistype',analysistype)
	% run the dynamic analysis
	!Opensees.exe ShearWall_model.tcl
    % read the results and plot thetaroof-dt 
    RoofDrift=dlmread(fullfile(FilesPath,'roofdrx.out'));figure; hold on
    plot(RoofDrift(:,1),RoofDrift(:,2),'linewidth',3)
	ylabel('roof drift','fontsize',LabelSize)
	grid on; box on; set(gca,'fontsize',FigureFontSize)
    % the figure will be stored in the ResultsPath/dynamic directory 
    DynamicPath=[ResultsPath,'/',analysistype];
    % create the directory if it doesnt exist
	if ~exist(DynamicPath, 'dir'); mkdir(DynamicPath);  end
	% save the figures in .fig and .png format
	saveas(gca,fullfile(DynamicPath,'rdr'))
	saveas(gca,fullfile(DynamicPath,'rdr'),'png')

elseif strcmpi(analysistype,'IDA')==1
    % run Incremental Dynamic Analysis (IDA) using the records defined in
    % the runIDA_Shear_Wall.m file. The records should be located inside
    % the recorddir.
    
   	% write the .tcl file with the analysis parameters. To run IDA analysis
   	% the analysistype in Opensees file should be set to dynamic
	write_MatIDA_paramfile(FilesPath,'Mat_anlsfile.tcl',0,...
        'analysistype','dynamic')
        
    % by default the model files and Opensees.exe file should be located 
    % in anlsdir parent folder. To avoid asking from the user to copy-paste
    % the files manually, we do it automatically using the following
    % function
    preIDA_ShearWall( anlsdir, FilesPath )
    
	% load the path of runIDA_Shear_Wall.m file
	cd(IDAPTpath)
	% Run the analysis! This analysis may last for a couple of hours so do
	% not worry about it! The output of this analysis is 
    % ShearWallIDAresults.sida.mat file found in anlsdir.
	% To post-process the results you can use
	% the PerformanceAseessment_PTwall.m file.
	runIDA_Shear_Wall(anlsdir,recorddir)
	
elseif strcmpi(analysistype,'plotSection')==1
	% plot the wall section. This option can be selected only if you first
	% run a pushover or dynamic analysis of the model so that the section
	% files have been created
    
    
    % Check if the section file exists
    if exist(['RCsectionRectfiberS1',num2str(Story),'.out'],'file')==2
		% You can plot the section of the shear wall. 
		% The name of the file is defined automatically based on the story
		% parameter that is defined earlier. If you want to change the
		% story to be plot you should modify the Story 
		SectionName=['RCsectionRectfiberS1',num2str(Story),'.out'];
		% Define the path where the section .out file is stored. By default
		% it will be saved in FilesPath path
		SectionFilePath=FilesPath;
		% Now plot the section. The "1.5" controls the size of steel rebars
        % of the figure. Change it to see the difference!
        figure; clf;
		plotSection(fullfile(SectionFilePath,SectionName),1.5);
		% As you can see, the reinforcement is concentrated in the middle
		% of the wall section and the section is discretized in fibers 
		% only in one direction. This can be done since we run a 2D model.  
		% You can change number of fibers or the location of steel rebars 
		% in ...\OpenseesModel\RCSect\RCsectionwallShearWall2.tcl file

        % save the figure inside Results\section path
        % create the directory if it doesnt exist
        SectionPath=[ResultsPath,'/section'];
        % create the directory if it doesnt exist
        if ~exist(SectionPath, 'dir'); mkdir(SectionPath);  end
        % save the figures in .fig and .png format
        saveas(gca,fullfile(SectionPath,'WallSection'))
        saveas(gca,fullfile(SectionPath,'WallSection'),'png')
    else
		error ('please run pushover or dynamic analysis first...')
    end
elseif strcmpi(analysistype,'plotModel')==1
    % plot the 2D model created in Opensees.
    
	% Define the path where the ShearWall.out file is stored. You
	% should first run any kind of analysis for this file to be
	% created. By default it is saved in the FilesPath folder
	EVfname=[FilesPath,'/ShearWall.out'];
    if exist(EVfname,'file')==2
        [FEnode,FEelt,FEfix,FEzero,FErigid,cdisps,eigv,SPcon,MPcon,...
            nloads,nmass,MQN,ndim]=read_opensees_modelfile(EVfname);
        figure
        clf
        plotframe2(FEnode,FEelt,FEfix,FEzero,FErigid,nloads,nmass,ndim,[1,1,0,0]);
        % save the figure of the model in Results/model path
         % create the directory if it doesnt exist
        ModelPath=[ResultsPath,'/model'];
        % create the directory if it doesnt exist
        if ~exist(ModelPath, 'dir'); mkdir(ModelPath);  end
        % save the figures in .fig and .png format
        saveas(gca,fullfile(ModelPath,'ShearWallModel'))
        saveas(gca,fullfile(ModelPath,'ShearWallModel'),'png')
        
    else
        error ('please run pushover or dynamic analysis first...')

    end
end

